from .singleton import Singleton


class ProgressionTypes(metaclass=Singleton):
    __progression_types = {
        "default": [
            {
                "name": "XpCapValue",
                "category": "Persistence",
                "kind": {"intValue": 10000},
                "mutatorId": "400274122",
            }
        ],
        "portal-standard": [
            {
                "name": "MasteryLevelCap",
                "category": "Persistence",
                "kind": {"intValue": 1},
                "mutatorId": "2587263936",
            },
            {
                "name": "AwardFilter",
                "category": "Persistence",
                "kind": {
                    "boolSparse": {
                        "sparseValues": [
                            {"index": 0, "value": None},
                            {"index": 1, "value": True},
                            {"index": 2, "value": None},
                            {"index": 3, "value": None},
                            {"index": 4, "value": None},
                            {"index": 5, "value": None},
                            {"index": 6, "value": None},
                            {"index": 7, "value": None},
                            {"index": 8, "value": None},
                            {"index": 9, "value": None},
                            {"index": 10, "value": None},
                            {"index": 11, "value": True},
                        ],
                        "defaultValue": True,
                    }
                },
                "mutatorId": "3878594831",
            },
            {
                "name": "PersistStats",
                "category": "Persistence",
                "kind": {"boolValue": False},
                "mutatorId": "877869937",
            },
            {
                "name": "ProgressionModeUIType",
                "category": "Persistence",
                "kind": {"intValue": 2},
                "mutatorId": "1058629035",
            },
            {
                "name": "XpCapValue",
                "category": "Persistence",
                "kind": {"intValue": 300},
                "mutatorId": "400274122",
            },
        ],
        "portal-featured": [
            {
                "name": "XpCapValue",
                "category": "Persistence",
                "kind": {"intValue": 10000},
                "mutatorId": "400274122",
            }
        ],
        "solo-coop": [
            {
                "name": "MasteryLevelCap",
                "category": "Persistence",
                "kind": {"intValue": 12},
                "mutatorId": "2587263936",
            },
            {
                "name": "AwardFilter",
                "category": "Persistence",
                "kind": {
                    "boolSparse": {
                        "sparseValues": [
                            {"index": 0, "value": None},
                            {"index": 1, "value": True},
                            {"index": 2, "value": True},
                            {"index": 3, "value": None},
                            {"index": 4, "value": True},
                            {"index": 5, "value": True},
                            {"index": 6, "value": None},
                            {"index": 7, "value": None},
                            {"index": 8, "value": None},
                            {"index": 9, "value": True},
                            {"index": 10, "value": True},
                            {"index": 11, "value": True},
                            {"index": 12, "value": None},
                        ],
                        "defaultValue": True,
                    }
                },
                "mutatorId": "3878594831",
            },
            {
                "name": "PersistStats",
                "category": "Persistence",
                "kind": {"boolValue": False},
                "mutatorId": "877869937",
            },
            {
                "name": "ProgressionModeUIType",
                "category": "Persistence",
                "kind": {"intValue": 1},
                "mutatorId": "1058629035",
            },
            {
                "name": "XpCapValue",
                "category": "Persistence",
                "kind": {"intValue": 100},
                "mutatorId": "400274122",
            },
        ],
        "portal-default": [
            {
                "name": "XpCapValue",
                "category": "Persistence",
                "kind": {"intValue": 10000},
                "mutatorId": "400274122",
            }
        ],
        "portal-unranked": [
            {
                "name": "AwardFilter",
                "category": "Persistence",
                "kind": {
                    "boolSparse": {
                        "sparseValues": [
                            {"index": 0, "value": None},
                            {"index": 1, "value": True},
                            {"index": 2, "value": None},
                            {"index": 3, "value": None},
                            {"index": 4, "value": None},
                            {"index": 5, "value": None},
                            {"index": 6, "value": None},
                            {"index": 7, "value": None},
                            {"index": 8, "value": None},
                            {"index": 9, "value": None},
                            {"index": 10, "value": None},
                            {"index": 11, "value": True},
                            {"index": 12, "value": None},
                            {"index": 13, "value": None},
                        ],
                        "defaultValue": True,
                    }
                },
                "mutatorId": "3878594831",
            },
            {
                "name": "MasteryLevelCap",
                "category": "Persistence",
                "kind": {"intValue": 1},
                "mutatorId": "2587263936",
            },
            {
                "name": "PersistStats",
                "category": "Persistence",
                "kind": {"boolValue": False},
                "mutatorId": "877869937",
            },
            {
                "name": "ProgressionModeUIType",
                "category": "Persistence",
                "kind": {"intValue": 2},
                "mutatorId": "1058629035",
            },
            {
                "name": "XpCapValue",
                "category": "Persistence",
                "kind": {"intValue": 300},
                "mutatorId": "400274122",
            },
        ],
        "portal-spotlight": [
            {
                "name": "XpCapValue",
                "category": "Persistence",
                "kind": {"intValue": 10000},
                "mutatorId": "400274122",
            }
        ],
    }

    async def updateProgression(self, progression_types: dict):
        self.__progression_types = progression_types

    async def get(self):
        return self.__progression_types


async def getProgressionType(item: str):
    progression_types = ProgressionTypes()
    progression_types_list = await progression_types.get()
    return progression_types_list.get(item, {})



import asyncio
import datetime
import logging
import re
import json

from global_mapping import bf2042 as BF2042
from global_mapping.readability.bf2042.languages.settingValues.main import (
    selectLanguage as settingsLanguage,
)
from global_mapping.readability.bf2042.saved_items.progresstion_types import (
    getProgressionType,
)
from global_mapping.readability.exceptions import NotFoundException
from global_mapping.readability.shared import format_percentage_value

logger = logging.getLogger("api")


async def settingTranslation(settings_translation: dict, settings: list[dict]):
    for setting in settings:
        for value in setting.get("values", []):
            try:
                value["readableSettingName"] = settings_translation[
                    value["settingName"]
                ]
            except KeyError:
                logger.warning(f"new setting, bf2042: {value.get('settingName', '')}")
                value["readableSettingName"] = ""


async def extraMapInfo(maps):
    for current_map in maps:
        internal_map_name = current_map.get("mapname", "")
        try:
            current_map["mapname"] = BF2042.MAPS[internal_map_name]
            current_map["image"] = BF2042.MAP_PICTURES[internal_map_name]
        except KeyError:
            logger.warning(f"new map, bf2042: {internal_map_name}")
            current_map["mapname"] = internal_map_name
            current_map["image"] = ""
        try:
            current_map["mode"] = BF2042.MODES[current_map["mode"]]
        except KeyError:
            logger.warning(f"new mode, bf2042: {current_map.get('mode', '')}")
            current_map["mode"] = current_map.get("mode", "")


async def serverList(servers):
    for server in servers:
        internal_map_name = server.get("currentMap", "")
        internal_mode_name = server.get("mode", "")
        internal_region_name = server.get("region", "")
        server["regionId"] = internal_region_name
        # compat with bf1:
        try:
            server["ownerId"] = f"{server['owner']['personaId']}"
        except KeyError:
            server["ownerId"] = ""

        try:
            server["currentMap"] = BF2042.MAPS[internal_map_name]
            server["url"] = BF2042.MAP_PICTURES[internal_map_name]
        except KeyError:
            logger.warning(f"new map, bf2042: {internal_map_name}")
            server["currentMap"] = internal_map_name
            server["url"] = ""
        try:
            server["mode"] = BF2042.MODES[internal_mode_name]
            server["smallMode"] = BF2042.SMALLMODES[internal_mode_name]
        except KeyError:
            logger.warning(f"new mode, bf2042: {internal_mode_name}")
            server["mode"] = internal_mode_name
            server["smallMode"] = ""
        try:
            server["owner"]["platform"] = BF2042.PLATFORM[server["owner"]["platformId"]]
        except KeyError:
            server_owner = server.get("owner")
            logger.warning(f"new platform, bf2042: {server_owner.get('platformId', 0)}")
            server["owner"]["platform"] = server_owner.get("platformId", 0)
        try:
            server["region"] = BF2042.REGIONS[internal_region_name]
        except KeyError:
            logger.warning(f"new region, bf2042: {internal_region_name}")
            server["region"] = ""
    return servers


async def detailedServer(server, server_id, lang: str):
    server_info = server.get("server_info", {})
    map_rotation = server_info.get("rotation", {})

    try:
        current_map = map_rotation.get("maps", [])[server_info.get("currentMapId", 0)]
    except IndexError:
        current_map = None

    if (
        current_map is not None
        and BF2042.MAP_FACTIONS.get(current_map.get("mapname", "")) is not None
    ):
        teams = BF2042.MAP_FACTIONS.get(
            current_map.get("mapname", ""),
            [{"name": "team 1", "image": None}, {"name": "team 2", "image": None}],
        )
        server_info["teams"] = {
            "teamOne": BF2042.FACTIONS.get(teams[0], {"name": "team 1", "image": None}),
            "teamTwo": BF2042.FACTIONS.get(teams[1], {"name": "team 2", "image": None}),
        }
    else:
        server_info["teams"] = {
            "teamOne": {"name": "team 1", "image": None},
            "teamTwo": {"name": "team 2", "image": None},
        }

    # mapnames and modes
    await extraMapInfo(map_rotation.get("maps", {}))

    server_info["currentMap"] = current_map.get("mapname", "")
    server_info["currentMapImage"] = current_map.get("image", "")

    # current map
    server_info["rotation"] = server_info.get("rotation", {}).get("maps", [])
    internal_region_name = server_info.get("pingSite", "")

    # platform name
    if server_info.get("owner", None) is not None:
        server_owner = server_info.get("owner", {})
        server_owner["id"] = server_owner.get("personaId", 0)
        try:
            server_owner["platform"] = BF2042.PLATFORM[server_owner["platformId"]]
        except KeyError:
            logger.warning(
                f"new platform, bf2042: {server['server_info']['owner']['platformId']}"
            )
            server_owner["platform"] = server_owner["platformId"]
    if server_info.get("configCreator", None) is not None:
        config_creator = server_info.get("configCreator", {})
        config_creator["id"] = config_creator.get("personaId", 0)
        try:
            config_creator["platform"] = BF2042.PLATFORM[config_creator["platformId"]]
        except KeyError:
            logger.warning(
                f"new platform, bf2042: {config_creator.get('platformId', 0)}"
            )
            config_creator["platform"] = config_creator.get("platformId", 0)

    # regionname
    try:
        server_info["region"] = BF2042.SHORT_REGIONS[internal_region_name]
    except KeyError:
        logger.warning(f"new region, bf2042: {internal_region_name}")
        server_info["region"] = ""

    extra_info = server_info.get("serverInfo", {})

    # to make it closer to bf1:
    server_info["prefix"] = extra_info.get("serverName", "")
    server_info["description"] = extra_info.get("serverDescription", "")
    server_info["serverId"] = server_id

    settings_translation = await settingsLanguage(lang)
    # server settings
    await settingTranslation(settings_translation, server_info.get("settings", {}))

    regexp = re.compile(r"^ID_.*_DESC$")
    if extra_info is not None:
        if regexp.search(extra_info.get("configDescription", "")):
            extra_info["configNameTranslation"] = settings_translation.get(
                extra_info.get("configName", ""), None
            )
            extra_info["configDescriptionTranslation"] = settings_translation.get(
                extra_info.get("configDescription", ""), None
            )
        else:
            extra_info["configNameTranslation"] = ""
            extra_info["configDescriptionTranslation"] = ""

    return server


async def tagTranslation(settings_translation: dict, tags: list[dict]):
    for tag in tags:
        metadata = tag.get("metadata", None)
        if metadata is not None:
            for translation in metadata["translations"]:
                try:
                    translation["localizedText"] = settings_translation.get(
                        translation.get("translationId", ""), ""
                    )
                    translation["localizedTextDesc"] = settings_translation.get(
                        f"{translation.get('translationId', '')}_DESC", ""
                    )
                except KeyError:
                    logger.warning(
                        f"blueprint assetCategories, bf2042: {translation['translationId']}"
                    )
                    translation["localizedText"] = ""
                    translation["localizedTextDesc"] = ""
            for resource in metadata["resources"]:
                try:
                    resource["location"]["url"] = (
                        resource["location"]
                        .get("url", "")
                        .replace(
                            "[BB_PREFIX]",
                            "https://eaassets-a.akamaihd.net/battlelog/battlebinary",
                        )
                    )
                except KeyError:
                    pass
                except AttributeError:
                    pass


async def players(player_list):
    for player in player_list.get("results", []):
        player["platformId"] = player.get("platform", 0)
        player["platform"] = BF2042.STATS_PLATFORM.get(player.get("platform", 0), "pc")
    return player_list


async def playground(current_playground, lang: str, playground_web_data: bytes):
    if playground_web_data is not None:
        try:
            decoded = playground_web_data.decode("utf8", "ignore")
            item = re.findall('{".*?"}', decoded)
            web_data = json.loads(item[0])
        except:
            web_data = {"mainWorkspace": "", "variables": ""}
    else:
        web_data = playground_web_data

    settings_translation = await settingsLanguage(lang)
    # server settings

    tasks = []

    if current_playground["playground"].get("tag", None) is not None:
        tasks.append(
            tagTranslation(
                settings_translation, current_playground["playground"]["tag"]
            )
        )
    if current_playground["playground"].get("originalPlayground", None) is not None:
        tasks.append(
            extraMapInfo(
                current_playground["playground"]["originalPlayground"]["mapRotation"][
                    "maps"
                ]
            )
        )
    if current_playground["playground"].get("validatedPlayground", None) is not None:
        tasks.append(
            extraMapInfo(
                current_playground["playground"]["validatedPlayground"]["mapRotation"][
                    "maps"
                ]
            )
        )
    await asyncio.gather(*tasks)

    if current_playground["playground"].get("progressionMode", None) is not None:
        current_playground["playground"]["progressionMode"][
            "progressibles"
        ] = await getProgressionType(
            current_playground["playground"]["progressionMode"]["value"]
        )

    current_playground["playground"]["blocklyData"] = web_data
    return current_playground["playground"]


async def blueprintTranslation(settingsTranslation: dict, settings: list[dict]):
    for translation in settings:
        try:
            translation["localizedText"] = settingsTranslation[
                translation["translationId"]
            ]
        except KeyError:
            logger.warning(
                f"blueprint new settings, bf2042: {translation.get('translationId', '')}"
            )
            translation["localizedText"] = ""


async def gameDataTranslation(settingsTranslation: dict, settings: list[dict]):
    for setting in settings:
        await blueprintTranslation(
            settingsTranslation, setting["metadata"]["translations"]
        )


async def blueprint(blueprintData, lang: str):
    settings_translation = await settingsLanguage(lang)

    for blueprint_data in blueprintData["blueprint"]:
        tasks = [
            blueprintTranslation(
                settings_translation, blueprint_data["metadata"]["translations"]
            ),
            gameDataTranslation(
                settings_translation, blueprint_data["availableGameData"]["maps"]
            ),
            gameDataTranslation(
                settings_translation, blueprint_data["availableGameData"]["mutators"]
            ),
            tagTranslation(
                settings_translation,
                blueprint_data["availableGameData"]["assetCategories"]["tags"],
            ),
        ]
        for available_tag in blueprint_data["availableTags"]:
            tasks.append(
                blueprintTranslation(
                    settings_translation, available_tag["metadata"]["translations"]
                )
            )

        await asyncio.gather(*tasks)

    return blueprintData


async def getBackground(resources: list[dict]):
    for resource in resources:
        if (
            resource.get("url", None) is not None
            and resource["url"].get("value", None) is not None
        ):
            resource["url"]["value"] = resource["url"]["value"].replace(
                "[BB_PREFIX]", "https://eaassets-a.akamaihd.net/battlelog/battlebinary"
            )


async def available_tags(tag_data, lang: str):
    tag_translation = await settingsLanguage(lang)
    for tag in tag_data.get("availableTags", []):
        await blueprintTranslation(tag_translation, tag["metadata"]["translations"])
    return tag_data


async def getBackgroundSubitem(resources: list[dict]):
    for resource in resources:
        if (
            resource.get("item", None) is not None
            and resource["item"].get("string", None) is not None
        ):
            resource["item"]["string"] = resource["item"]["string"].replace(
                "[BB_PREFIX]", "https://eaassets-a.akamaihd.net/battlelog/battlebinary"
            )


async def scheduled_collections(collections_data, lang: str):
    collection_translations = await settingsLanguage(lang)
    tasks = []
    for collection in collections_data.get("collectionList", []):
        for value in collection.get("collectionValues", []):
            for mix in value.get("mixes", []):
                tasks.append(
                    blueprintTranslation(
                        collection_translations, mix["assets"]["item"]["translations"]
                    )
                )
                tasks.append(getBackgroundSubitem(mix["assets"]["item"]["info"]))
                if mix["mixInfo"] is not None:
                    tasks.append(
                        blueprintTranslation(
                            collection_translations,
                            mix["mixInfo"]["meta"]["item"]["translations"],
                        )
                    )
            if value["main"] is not None:
                tasks.append(
                    blueprintTranslation(
                        collection_translations,
                        value["main"]["subEntry"]["translations"],
                    )
                )
                tasks.append(getBackground(value["main"]["subEntry"]["resource"]))
            if value["secondary"] is not None:
                tasks.append(
                    blueprintTranslation(
                        collection_translations,
                        value["secondary"]["menuEntry"]["subEntry"]["translations"],
                    )
                )
                tasks.append(
                    getBackground(
                        value["secondary"]["menuEntry"]["subEntry"]["resource"]
                    )
                )
    await asyncio.gather(*tasks)
    return collections_data


async def mix_info(mix_data):
    for mix in mix_data.get("mixes", []):
        await extraMapInfo(mix["mapRotation"]["maps"])
    return mix_data


async def get_offers(offer_data, lang: str):
    offer_translation = await settingsLanguage(lang)
    for value in offer_data.get("items", []):
        value["readableTitle"] = offer_translation.get(value.get("title", ""), "")
        value["readableDescription"] = offer_translation.get(
            value.get("description", ""), ""
        )
        for name, art_item in value.get("art", {}).items():
            if art_item is not None:
                value["art"][name]["art"] = art_item.get("art", "").replace(
                    "[BB_PREFIX]",
                    "https://eaassets-a.akamaihd.net/battlelog/battlebinary",
                )

    for bundle in offer_data.get("bundles", []):
        for featured in bundle.get("featured", []):
            for layout in featured.get("layout", []):
                layout["readableTitle"] = offer_translation.get(
                    layout.get("title", ""), ""
                )
        battlepass_banner = bundle.get("battlepassBanner", {})
        if battlepass_banner is not None:
            battlepass_banner["url"] = battlepass_banner.get("url", "").replace(
                "[BB_PREFIX]", "https://eaassets-a.akamaihd.net/battlelog/battlebinary"
            )
            battlepass_banner["localizedText"] = offer_translation.get(
                battlepass_banner.get("translationId", ""), ""
            )

    return offer_data


async def get_weapons(stats_dict: dict, constant: dict, format_values: bool = True):
    weapons = []
    for _id, extra in constant.items():
        kills = stats_dict.get(f"kw_wp_{_id}", 0)
        damage = stats_dict.get(f"dmg_wp_{_id}", 0)
        headshots = stats_dict.get(f"hsw_wp_{_id}", 0)
        shots_hit = stats_dict.get(f"shw_wp_{_id}", 0)
        shots_fired = stats_dict.get(f"sfw_wp_{_id}", 0)
        seconds = stats_dict.get(f"tp_wp_{_id}", 0)

        try:
            accuracy = round((shots_hit / shots_fired) * 100, 2)
        except ZeroDivisionError:
            accuracy = 0.0

        try:
            headshot_percentage = round(headshots / kills * 100, 2)
        except ZeroDivisionError:
            headshot_percentage = 0.0

        try:
            kills_per_minute = round(kills / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            kills_per_minute = 0.0

        try:
            damage_per_minute = round(damage / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            damage_per_minute = 0.0

        try:
            hits_per_kill = round(shots_hit / kills, 2)
        except ZeroDivisionError:
            hits_per_kill = 0.0

        weapons.append(
            {
                **extra,
                "id": _id,
                "kills": kills,
                "damage": damage,
                "bodyKills": stats_dict.get(f"bkw_wp_{_id}", 0),
                "headshotKills": headshots,
                "hipfireKills": stats_dict.get(f"hfkw_wp_{_id}", 0),
                "multiKills": stats_dict.get(f"mkw_wp_{_id}", 0),
                "accuracy": format_percentage_value(accuracy, format_values),
                "killsPerMinute": kills_per_minute,
                "damagePerMinute": damage_per_minute,
                "headshots": format_percentage_value(
                    headshot_percentage, format_values
                ),
                "hitVKills": hits_per_kill,
                "shotsHit": shots_hit,
                "shotsFired": shots_fired,
                "scopedKills": stats_dict.get(f"adskw_wp_{_id}", 0),
                "spawns": stats_dict.get(f"spawns_wp_{_id}", 0),
                "timeEquipped": seconds,
            }
        )
    return weapons


async def get_vehicles(stats_dict: dict, constant: dict):
    vehicles = []
    for _id, extra in constant.items():
        kills = stats_dict.get(f"kw_vh_{_id}", 0)
        seconds = stats_dict.get(f"tp_vh_{_id}", 0)
        try:
            kills_per_minute = round(kills / (seconds / 60), 2)
        except ZeroDivisionError:
            kills_per_minute = 0
        vehicles.append(
            {
                **extra,
                "id": _id,
                "kills": kills,
                "killsPerMinute": kills_per_minute,
                "damage": stats_dict.get(f"dmg_vh_{_id}", 0),
                "spawns": stats_dict.get(f"spawns_vh_{_id}", 0),
                "roadKills": stats_dict.get(f"roadkills_vh_{_id}", 0),
                "passengerAssists": stats_dict.get(f"assists_p_vh_{_id}", 0),
                "multiKills": stats_dict.get(f"mkw_vh_{_id}", 0),
                "distanceTraveled": stats_dict.get(f"disttrv_vh_{_id}", 0),
                "driverAssists": stats_dict.get(f"assists_d_vh_{_id}", 0),
                "vehiclesDestroyedWith": stats_dict.get(f"vdw_vh_{_id}", 0),
                "assists": stats_dict.get(f"assists_vh_{_id}", 0),
                "callIns": stats_dict.get(f"callins_vh_{_id}", 0),
                "damageTo": stats_dict.get(f"dmgTo_vh_{_id}", 0),
                "destroyed": stats_dict.get(f"vd_vh_{_id}", 0),
                # "idk": stats_dict.get(f"eor_vh_{_id}"),
                "timeIn": seconds,
            }
        )
    return vehicles


async def get_classes(stats_dict: dict):
    kits = []
    for kit_id, extra in BF2042.CLASSES.items():
        kills = stats_dict.get(f"kw_char_{kit_id}", 0)
        deaths = stats_dict.get(f"deaths_char_{kit_id}", 0)
        seconds = stats_dict.get(f"tp_char_{kit_id}", 0)
        try:
            kill_death = round(kills / deaths, 2)
        except ZeroDivisionError:
            kill_death = 0.0
        try:
            kills_per_minute = round(kills / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            kills_per_minute = 0.0
        kits.append(
            {
                **extra,
                "id": kit_id,
                "kills": kills,
                "deaths": deaths,
                "kpm": kills_per_minute,
                "killDeath": kill_death,
                "spawns": stats_dict.get(f"spawns_char_{kit_id}", 0),
                "revives": stats_dict.get(f"revives_char_{kit_id}", 0),
                "assists": stats_dict.get(f"assists_char_{kit_id}", 0),
                "hazardZoneStreaks": stats_dict.get(f"hz_streak_char_{kit_id}", 0),
                "secondsPlayed": seconds,
            }
        )
    return kits


async def get_gamemodes(stats_dict: dict, format_values: bool = False):
    gamemodes = []
    for _id, extra in BF2042.STAT_GAMEMODE.items():
        wins = stats_dict.get(f"wins_gm_{_id}", 0)
        losses = stats_dict.get(f"losses_gm_{_id}", 0)
        kills = stats_dict.get(f"kills_gm_{_id}", 0)
        seconds = stats_dict.get(f"tp_gm_{_id}", 0)
        try:
            kills_per_minute = round(kills / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            kills_per_minute = 0.0
        try:
            win_percent = round(wins / (wins + losses) * 100, 2)
        except ZeroDivisionError:
            win_percent = 0.0

        gamemodes.append(
            {
                **extra,
                "id": _id,
                "kills": kills,
                "assists": stats_dict.get(f"assists_gm_{_id}", 0),
                "revives": stats_dict.get(f"revives_gm_{_id}", 0),
                "bestSquad": stats_dict.get(f"bestsquad_gm_{_id}", 0),
                "wins": wins,
                "losses": losses,
                "mvp": stats_dict.get(f"mvp_gm_{_id}", 0),
                "matches": stats_dict.get(f"matches_gm_{_id}", 0),
                "sectorDefend": stats_dict.get(f"sectordef_gm_{_id}", 0),
                "objectivesArmed": stats_dict.get(f"obj_armed_gm_{_id}", 0),
                "objectivesDisarmed": stats_dict.get(f"obj_disarmed_gm_{_id}", 0),
                "objectivesDefended": stats_dict.get(f"obj_defended_gm_{_id}", 0),
                "objectivesCaptured": stats_dict.get(f"obj_captured_gm_{_id}", 0),
                "objectivesDestroyed": stats_dict.get(f"obj_destroyed_gm_{_id}", 0),
                "objetiveTime": stats_dict.get(f"obj_time_gm_{_id}", 0),
                "kpm": kills_per_minute,
                "winPercent": format_percentage_value(win_percent, format_values),
                "secondsPlayed": seconds,
            }
        )
    return gamemodes


async def get_maps(stats_dict: dict, format_values: bool = False):
    maps = []
    for _id, extra in BF2042.STAT_MAPS.items():
        wins = stats_dict.get(f"wins_lvl_{_id}", 0)
        losses = stats_dict.get(f"losses_lvl_{_id}", 0)
        try:
            win_percent = round(wins / (wins + losses) * 100, 2)
        except ZeroDivisionError:
            win_percent = 0.0

        maps.append(
            {
                **extra,
                "id": _id,
                "wins": wins,
                "losses": losses,
                "matches": stats_dict.get(f"matches_lvl_{_id}", 0),
                "winPercent": format_percentage_value(win_percent, format_values),
                "secondsPlayed": stats_dict.get(f"tp_lvl_{_id}", 0),
            }
        )
    return maps


async def get_gadgets(stats_dict: dict):
    gadgets = []
    for _id, extra in BF2042.GADGETS.items():
        damage = stats_dict.get(f"dmg_gad_{_id}", 0)
        kills = stats_dict.get(f"kw_gad_{_id}", 0)
        seconds = stats_dict.get(f"tp_gad_{_id}", 0)

        try:
            kills_per_minute = round(kills / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            kills_per_minute = 0.0

        try:
            damage_per_minute = round(damage / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            damage_per_minute = 0.0

        gadgets.append(
            {
                **extra,
                "id": _id,
                "kills": kills,
                "spawns": stats_dict.get(f"spawns_gad_{_id}", 0),
                "damage": damage,
                "uses": stats_dict.get(f"uses_gad_{_id}", 0),
                "multiKills": stats_dict.get(f"mkw_gad_{_id}", 0),
                "vehiclesDestroyedWith": stats_dict.get(f"vdw_gad_{_id}", 0),
                "kpm": kills_per_minute,
                "dpm": damage_per_minute,
                "secondsPlayed": seconds,
            }
        )
    return gadgets


async def get_main_stats(stats_dict: dict, format_values: bool = True):
    shots_fired = stats_dict.get("shots_fired_total", 0)
    shots_hit = stats_dict.get("shots_hit_total", 0)
    wins = stats_dict.get("won_match_total", 0)
    losses = stats_dict.get("lost_match_total", 0)
    kills = stats_dict.get("Kills_Total", 0)
    deaths = stats_dict.get("deaths_total", 0)
    seconds = stats_dict.get("tp_gm_all", 0)
    damage = stats_dict.get("dmg_total", 0)
    human_kills = stats_dict.get("human_kills_total", 0)
    headshot_amount = stats_dict.get("kills_Headshots_Total", 0)
    matches_played = stats_dict.get("played_match_total", 0)
    try:
        accuracy = round((shots_hit / shots_fired) * 100, 2)
    except ZeroDivisionError:
        accuracy = 0.0
    try:
        win_percent = round(wins / (wins + losses) * 100, 2)
    except ZeroDivisionError:
        win_percent = 0.0
    try:
        headshots = round((headshot_amount / kills) * 100, 2)
    except ZeroDivisionError:
        headshots = 0.0
    try:
        kill_death = round(kills / deaths, 2)
        infantry_kill_death = round(human_kills / deaths, 2)
    except ZeroDivisionError:
        kill_death = 0.0
        infantry_kill_death = 0.0
    try:
        kills_per_minute = round(kills / (seconds / 60), 2)
        damage_per_minute = round(damage / (seconds / 60), 2)
    except (ZeroDivisionError, KeyError):
        kills_per_minute = 0.0
        damage_per_minute = 0.0

    try:
        kills_per_match = round(kills / matches_played, 2)
        damage_per_match = round(damage / matches_played, 2)
    except (ZeroDivisionError, KeyError):
        kills_per_match = 0.0
        damage_per_match = 0.0

    try:
        human_precentage = round((human_kills / kills) * 100, 2)
    except ZeroDivisionError:
        human_precentage = 0.0

    return {
        "humanPrecentage": format_percentage_value(human_precentage, format_values),
        "kills": kills,
        "deaths": deaths,
        "wins": wins,
        "loses": losses,
        "killsPerMinute": kills_per_minute,
        "damagePerMinute": damage_per_minute,
        "killsPerMatch": kills_per_match,
        "damagePerMatch": damage_per_match,
        "headShots": headshot_amount,
        "winPercent": format_percentage_value(win_percent, format_values),
        "headshots": format_percentage_value(headshots, format_values),
        "killDeath": kill_death,
        "infantryKillDeath": infantry_kill_death,
        "damage": damage,
        "timePlayed": str(datetime.timedelta(seconds=stats_dict.get("tp_gm_all", 0))),
        "accuracy": format_percentage_value(accuracy, format_values),
        "revives": stats_dict.get("Revives_Teammates_Total", 0),
        "heals": stats_dict.get("Dmg_Healed_Total", 0),
        "resupplies": stats_dict.get("Resupply_Teammates_Total", 0),
        "repairs": stats_dict.get("Veh_RepairedHP_Total", 0),
        "squadmateRevive": stats_dict.get("Revives_Squadmates_Total", 0),
        "squadmateRespawn": stats_dict.get("Respawns_Squadmates_Total", 0),
        "thrownThrowables": stats_dict.get("thrown_throwables_total", 0),
        "inRound": {
            "revives": stats_dict.get("revives_inround_cb", 0),
            "resupplies": stats_dict.get("resupply_teammates_inround_cb", 0),
            "squadmateRevive": stats_dict.get("revives_squadmates_inround_cb", 0),
            "spotAssists": stats_dict.get("spot_assist_inround_cb", 0),
            "thrownThrowables": stats_dict.get("thrown_throwables_inround_cb", 0),
            "playerTakeDowns": stats_dict.get("players_taken_down_inround_cb", 0),
            "gadgetsDestoyed": stats_dict.get("disable_gadget_inround_cb", 0),
        },
        "gadgetsDestoyed": stats_dict.get("destroy_gadget_total", 0),
        "callIns": stats_dict.get("call_ins_total", 0),
        "playerTakeDowns": (stats_dict.get("Players_Taken_Down_Total", 0)),
        "matchesPlayed": matches_played,
        "secondsPlayed": seconds,
        "dividedSecondsPlayed": {
            "flying": stats_dict.get("flying_time_total", 0),
            "driving": stats_dict.get("driving_time_total", 0),
        },
        "bestSquad": stats_dict.get("best_squad_total", 0),
        "teammatesSupported": stats_dict.get("supported_teammates_total", 0),
        "saviorKills": stats_dict.get("savior_kills_total", 0),
        "shotsFired": shots_fired,
        "shotsHit": shots_hit,
        "killAssists": stats_dict.get("assists_total", 0),
        "vehiclesDestroyed": stats_dict.get("Destroyed_Veh_Total", 0),
        "enemiesSpotted": stats_dict.get("Spotted_Enemies_Total", 0),
        "mvp": stats_dict.get("mvp_total", 0),
        "dividedKills": {
            "ads": stats_dict.get("Kills_ADS_Total", 0),
            "grenades": stats_dict.get("Kills_Grenade_Total", 0),
            "hipfire": stats_dict.get("Kills_Hipfire_Total", 0),
            "longDistance": stats_dict.get("Kills_LongDist_Total", 0),
            "melee": stats_dict.get("Kills_Melee_Total", 0),
            "multiKills": stats_dict.get("Kills_Multi_Total", 0),
            "parachute": stats_dict.get("Kills_Parachute_Total", 0),
            "passenger": stats_dict.get("Kills_Psgr_Total", 0),
            "vehicle": stats_dict.get("Kills_wVeh_Total", 0),
            "roadkills": stats_dict.get("kill_road_total", 0),
            "ai": stats_dict.get("AI_kills_total", 0),
            "human": human_kills,
            "turret": stats_dict.get("kills_turret_total", 0),
            "ranger": stats_dict.get("kills_ranger_total", 0),
            "weapons": {
                "BAR": stats_dict.get("kills_bar_total", 0),
                "SMG": stats_dict.get("kills_smg_total", 0),
                "LMG": stats_dict.get("kills_lmg_total", 0),
                "DMR": stats_dict.get("kills_dmr_total", 0),
                "Sidearm": stats_dict.get("kills_sa_total", 0),
                "Crossbows": stats_dict.get("kills_cb_total", 0),
                "Shotguns": stats_dict.get("kills_sg_total", 0),
                "Assault Rifles": stats_dict.get("kills_sg_total", 0),
            },
            "inRound": {
                "total": stats_dict.get("kills_inround_cb", 0),
                "turret": stats_dict.get("kills_turret_inround_cb", 0),
                "killsAndAssists": stats_dict.get("kills_and_assists_inround_cb", 0),
                "drone": stats_dict.get("kills_drone_inround_cb", 0),
                "grenade": stats_dict.get("kills_grenade_inround_cb", 0),
                "headshots": stats_dict.get("kills_headshots_inround_cb", 0),
                "hipfire": stats_dict.get("kills_hipfire_inround_cb", 0),
                "longDistance": stats_dict.get("kills_longdist_inround_cb", 0),
                "melee": stats_dict.get("kills_melee_inround_cb", 0),
                "multiKills": stats_dict.get("kills_multi_inround_cb", 0),
                "parachute": stats_dict.get("kills_parachute_inround_cb", 0),
                "passenger": stats_dict.get("kills_psgr_inround_cb", 0),
                "weapons": {
                    "Sidearm": stats_dict.get("kills_sa_inround_cb", 0),
                    "BAR": stats_dict.get("kills_bar_inround_cb", 0),
                    "SMG": stats_dict.get("kills_smg_inround_cb", 0),
                    "AR": stats_dict.get("kills_ar_inround_cb", 0),
                    "DMR": stats_dict.get("kills_dmr_inround_cb", 0),
                },
            },
        },
        "devidedDamage": {
            "explosion": stats_dict.get("dmg_expl_total", 0),
            "passenger": stats_dict.get("dmg_psgr_total", 0),
            "vehicleDriver": stats_dict.get("dmg_vehdriver_total", 0),
            "toVehicle": stats_dict.get("dmgto_veh_total", 0),
            "inRound": {
                "passenger": stats_dict.get("dmg_psgr_inround_cb", 0),
                "explosion": stats_dict.get("dmg_expl_inround_cb", 0),
                "toVehicle": stats_dict.get("dmgto_veh_inround_cb", 0),
                "asVehicle": stats_dict.get("dmgwith_veh_inround_cb", 0),
            },
        },
        "devidedAssists": {
            "driver": stats_dict.get("Assists_Driver_Total", 0),
            "passenger": stats_dict.get("Assists_Psgr_Total", 0),
            "spot": stats_dict.get("Assists_Spot_Total", 0),
            "pilot": stats_dict.get("assists_pilot_total", 0),
            "ai": stats_dict.get("AI_assists_total", 0),
            "inRound": {
                "total": stats_dict.get("assist_inround_cb", 0),
                "passenger": stats_dict.get("assists_psgr_inround_cb", 0),
                "pilot": stats_dict.get("assists_pilot_inround_cb", 0),
            },
        },
        "distanceTraveled": {
            "foot": stats_dict.get("distrav_foot_total", 0),
            "grapple": stats_dict.get("distrav_grapple_total", 0),
            "passenger": stats_dict.get("distrav_psgr_total", 0),
            "vehicle": stats_dict.get("distrav_veh_total", 0),
        },
        "sector": {
            "captured": stats_dict.get("sect_cap_total", 0),
            "defended": stats_dict.get("sect_def_total", 0),
        },
        "objective": {
            "time": {
                "total": stats_dict.get("obj_time_total", 0),
                "attacked": stats_dict.get("attobj_time_Total", 0),
                "defended": stats_dict.get("defobj_time_Total", 0),
            },
            "armed": stats_dict.get("obj_arm_total", 0),
            "captured": stats_dict.get("cap_obj_total", 0),
            "neutralized": stats_dict.get("neutralized_obj_total", 0),
            "defused": stats_dict.get("obj_defuse_total", 0),
            "destroyed": stats_dict.get("destroyed_obj_total", 0),
            "inRound": {
                "time": stats_dict.get("obj_time_inround_cb", 0),
                "armed": stats_dict.get("obj_arm_inround_cb", 0),
                "captured": stats_dict.get("cap_obj_inround_cb", 0),
                "neutralized": stats_dict.get("cap_neut_obj_inround_cb", 0),
                "defused": stats_dict.get("obj_defuse_inround_cb", 0),
                "destroyed": stats_dict.get("obj_dest_inround_cb", 0),
            },
        },
        "XP": (
            {
                "total": stats_dict.get("xp_all", 0),
                "performance": stats_dict.get("xp_performance", 0),
                "ribbons": {
                    "total": stats_dict.get("xp_ribbons", 0),
                    "squad": stats_dict.get("xp_ribbons_Squad", 0),
                    "combat": stats_dict.get("xp_ribbons_combat", 0),
                    "intel": stats_dict.get("xp_ribbons_intel", 0),
                    "objective": stats_dict.get("xp_ribbons_objective", 0),
                    "support": stats_dict.get("xp_ribbons_support", 0),
                },
            },
        ),
        "squadOrders": {
            "attack": {
                "placed": stats_dict.get("sqorder_attack_placed_total", 0),
                "completed": stats_dict.get("sqorder_attack_comp_total", 0),
            },
            "defence": {
                "placed": stats_dict.get("sqorder_defend_placed_total", 0),
                "completed": stats_dict.get("sqorder_defend_comp_total", 0),
            },
            "placed": stats_dict.get("sqorder_placed_total", 0),
            "completed": stats_dict.get("sqorder_comp_total", 0),
        },
        "intel": {
            "squad": {
                "extracted": stats_dict.get("intel_ext_squad_total", 0),
            },
            "pickup": {
                "squad": stats_dict.get("intel_pickup_squad_total", 0),
                "total": stats_dict.get("intel_pickup_total", 0),
                "gamemodes": {
                    "all": stats_dict.get("intel_pickup_gm_all", 0),
                    "hazardZone": stats_dict.get("intel_pickup_gm_hz", 0),
                },
            },
            "extracted": stats_dict.get("intel_ext_total", 0),
        },
        "seasons": {
            "1": {
                "kills": stats_dict.get("season1_kills", 0),
                "deaths": stats_dict.get("season1_death", 0),
                "wins": stats_dict.get("season1_matches_won", 0),
                "loses": stats_dict.get("season1_rounds_played", 0)
                - stats_dict.get("season1_matches_won", 0),
                "roundsPlayed": stats_dict.get("season1_rounds_played", 0),
                "heals": stats_dict.get("season1_teammate_healed", 0),
                "revives": stats_dict.get("season1_revives", 0),
                "resupplies": stats_dict.get("season1_teammate_resupplied", 0),
                "assists": stats_dict.get("season1_assists", 0),
                "headshots": stats_dict.get("season1_headshot_kills", 0),
                "enemiesSpotted": stats_dict.get("season1_enemies_spotted", 0),
                "vehiclesDestroyed": stats_dict.get("season1_veh_destroyed", 0),
                "ribbons": stats_dict.get("season1_ribbons", 0),
                "objective": {
                    "armed": stats_dict.get("season1_object_armed", 0),
                    "captured": stats_dict.get("season1_object_captured", 0),
                    "defused": stats_dict.get("season1_object_defused", 0),
                    "destroyed": stats_dict.get("season1_object_destroyed", 0),
                    "neutralized": stats_dict.get("season1_object_neutralized", 0),
                },
                "missionsCompleted": stats_dict.get("season1_missions_comp", 0),
                "intelExtracted": stats_dict.get("season1_intel_extracted", 0),
                "timesExtracted": stats_dict.get("season1_times_extracted", 0),
                "vehiclesHPRepaired": stats_dict.get("season1_veh_hp_repaired", 0),
            }
        },
    }


async def get_stats(
    data: dict,
    raw_loadout: dict = {},
    format_values: bool = True,
    multiple: bool = False,
):
    # get the level of each player to their id
    loadouts = {}
    for player in (
        raw_loadout.get("result", {"inventory": {"loadouts": []}})
        .get("inventory", {"loadouts": []})
        .get("loadouts", [])
    ):
        loadouts[player.get("player", {"personaId": 0}).get("personaId", 0)] = (
            player.get("level", 0)
        )

    result = []
    for player in data.get("playerStats", []):
        current_result = {}
        current_player = player.get("player", {"personaId": 0})
        stats_dict = {}
        for category in player.get("categories", []):
            for item in category.get("catFields", {"fields": []}).get("fields", []):
                stats_dict[item["name"]] = item["value"]

        current_result["hasResults"] = len(stats_dict) > 0

        tasks = []
        tasks.append(get_weapons(stats_dict, BF2042.WEAPONS, format_values))
        tasks.append(get_vehicles(stats_dict, BF2042.VEHICLES))
        tasks.append(get_weapons(stats_dict, BF2042.WEAPON_GROUPS, format_values))
        tasks.append(get_vehicles(stats_dict, BF2042.VEHICLE_GROUPS))
        tasks.append(get_classes(stats_dict))
        tasks.append(get_gamemodes(stats_dict, format_values))
        tasks.append(get_maps(stats_dict, format_values))
        tasks.append(get_gadgets(stats_dict))

        if multiple:
            platform_id = current_player.get("platformId", 0)
            current_result["id"] = current_player.get("personaId", 0)
            current_result["userId"] = current_player.get("nucleusId", 0)
            current_result["platform"] = BF2042.STATS_PLATFORM.get(
                current_player.get("platformId", 0), "pc"
            )
            current_result["platformId"] = 0 if platform_id is None else platform_id

        (
            current_result["weapons"],
            current_result["vehicles"],
            current_result["weaponGroups"],
            current_result["vehicleGroups"],
            current_result["classes"],
            current_result["gamemodes"],
            current_result["maps"],
            current_result["gadgets"],
        ) = await asyncio.gather(*tasks)

        kit_best_kills = 0
        best_kit = 0
        for kit in current_result["classes"]:
            if kit["kills"] > kit_best_kills:
                kit_best_kills = kit["kills"]
                best_kit = kit["characterName"]
        current_result["bestClass"] = best_kit

        current_result.update(await get_main_stats(stats_dict, format_values))
        # add fetched level
        current_result["level"] = loadouts.get(current_player.get("personaId", 0), 0)
        result.append(current_result)

    if len(result) < 1:
        raise NotFoundException("player not found")
    if len(result) > 1:
        return {"data": result}
    return result[0]



import asyncio
import datetime
from global_mapping import bf5 as BF5
from global_mapping.readability.shared import format_percentage_value


async def get_maps(stats_dict: dict):
    maps = []
    for _id, extra in BF5.STAT_MAPS.items():
        maps.append({**extra, "id": _id, "score": stats_dict.get(f"sc_lvl_{_id}", 0)})
    return maps


async def get_gamemodes(stats_dict: dict, format_values: bool = False):
    gamemodes = []
    for _id, extra in BF5.STAT_GAMEMODE.items():
        extra_str = BF5.STAT_GAMEMODE_EXTR.get(_id)
        wins = stats_dict.get(f"{_id}_wins", 0)
        losses = stats_dict.get(f"{_id}_losses", 0)

        try:
            win_percent = round(wins / (wins + losses) * 100, 2)
        except ZeroDivisionError:
            win_percent = 0.0

        gamemodes.append(
            {
                **extra,
                "id": _id,
                "score": stats_dict.get(f"sc_{extra_str}", 0),
                "wins": wins,
                "losses": losses,
                "winPercent": format_percentage_value(win_percent, format_values),
            }
        )
    return gamemodes


async def get_classes(stats_dict: dict, format_values: bool = True):
    kits = []
    for kit_id, extra in BF5.CLASSES.items():
        score = stats_dict.get(f"sc_{kit_id}", 0)
        kills = stats_dict.get(f"kit_{kit_id}_kills", 0)
        deaths = stats_dict.get(f"kit_{kit_id}_deaths", 0)
        seconds = stats_dict.get(f"kit_{kit_id}_time", 0)
        shots_hit = stats_dict.get(f"kit_{kit_id}_hits", 0)
        shots_fired = stats_dict.get(f"kit_{kit_id}_shots", 0)

        try:
            accuracy = round((shots_hit / shots_fired) * 100, 2)
        except ZeroDivisionError:
            accuracy = 0.0

        try:
            kill_death = round(kills / deaths, 2)
        except ZeroDivisionError:
            kill_death = 0.0
        try:
            kills_per_minute = round(kills / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            kills_per_minute = 0.0

        try:
            hits_per_kill = round(shots_hit / kills, 2)
        except ZeroDivisionError:
            hits_per_kill = 0.0

        try:
            score_per_minute = round(score / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            score_per_minute = 0.0
        except TypeError:
            score_per_minute = 0.0

        kits.append(
            {
                **extra,
                "id": kit_id,
                "score": score,
                "rank": stats_dict.get(f"rank_{kit_id}", 0),
                "kills": kills,
                "deaths": deaths,
                "kpm": kills_per_minute,
                "spm": score_per_minute,
                "killDeath": kill_death,
                "accuracy": format_percentage_value(accuracy, format_values),
                "hitVKills": hits_per_kill,
                "shotsHit": shots_hit,
                "shotsFired": shots_fired,
                "secondsPlayed": seconds,
            }
        )
    return kits


async def get_weapons(stats_dict: dict, constant: dict, format_values: bool = True):
    weapons = []
    for _id, extra in constant.items():
        kills = stats_dict.get(f"kw_{_id}", 0)
        score = stats_dict.get(f"scrw_{_id}", 0)
        headshots = stats_dict.get(f"hsw_{_id}", 0)
        shots_hit = stats_dict.get(f"shw_{_id}", 0)
        shots_fired = stats_dict.get(f"sfw_{_id}", 0)
        seconds = stats_dict.get(f"tpw_{_id}", 0)

        try:
            accuracy = round((shots_hit / shots_fired) * 100, 2)
        except ZeroDivisionError:
            accuracy = 0.0
        except TypeError:
            accuracy = 0.0

        try:
            headshot_percentage = round(headshots / kills * 100, 2)
        except ZeroDivisionError:
            headshot_percentage = 0.0
        except TypeError:
            headshot_percentage = 0.0

        try:
            kills_per_minute = round(kills / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            kills_per_minute = 0.0
        except TypeError:
            kills_per_minute = 0.0

        try:
            score_per_minute = round(score / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            score_per_minute = 0.0
        except TypeError:
            score_per_minute = 0.0

        try:
            hits_per_kill = round(shots_hit / kills, 2)
        except ZeroDivisionError:
            hits_per_kill = 0.0
        except TypeError:
            hits_per_kill = 0.0

        weapons.append(
            {
                **extra,
                "id": _id,
                "kills": kills,
                "score": score,
                "headshotKills": headshots,
                "accuracy": format_percentage_value(accuracy, format_values),
                "killsPerMinute": kills_per_minute,
                "scorePerMinute": score_per_minute,
                "headshots": format_percentage_value(
                    headshot_percentage, format_values
                ),
                "hitVKills": hits_per_kill,
                "shotsHit": shots_hit,
                "shotsFired": shots_fired,
                "timeEquipped": seconds,
            }
        )
    return weapons


async def get_vehicles(stats_dict: dict, constant: dict):
    vehicles = []
    for _id, extra in constant.items():
        kills = stats_dict.get(f"kw_{_id}", 0)
        seconds = stats_dict.get(f"tpw_{_id}", 0)
        try:
            kills_per_minute = round(kills / (seconds / 60), 2)
        except ZeroDivisionError:
            kills_per_minute = 0
        except TypeError:
            kills_per_minute = 0.0
        vehicles.append(
            {
                **extra,
                "id": _id,
                "kills": kills,
                "killsPerMinute": kills_per_minute,
                "vehiclesDestroyedWith": stats_dict.get(f"vdw_{_id}", 0),
            }
        )
    return vehicles


async def get_main_stats(stats_dict: dict, format_values: bool = True):
    shots_fired = stats_dict.get("shots", 0)
    shots_hit = stats_dict.get("hits", 0)
    wins = stats_dict.get("wins", 0)
    losses = stats_dict.get("losses", 0)
    kills = stats_dict.get("kills", 0)
    deaths = stats_dict.get("deaths", 0)
    headshot_amount = stats_dict.get("headshots", 0)
    matches_played = stats_dict.get("rounds_completed", 0)
    seconds = stats_dict.get("time", 0)
    score = stats_dict.get("sc_roundscore", 0)

    try:
        accuracy = round((shots_hit / shots_fired) * 100, 2)
    except ZeroDivisionError:
        accuracy = 0.0
    except TypeError:
        accuracy = 0.0

    try:
        win_percent = round(wins / (wins + losses) * 100, 2)
    except ZeroDivisionError:
        win_percent = 0.0
    except TypeError:
        win_percent = 0.0

    try:
        headshots = round((headshot_amount / kills) * 100, 2)
    except ZeroDivisionError:
        headshots = 0.0
    except TypeError:
        headshots = 0.0

    try:
        kill_death = round(kills / deaths, 2)
    except ZeroDivisionError:
        kill_death = 0.0
    except TypeError:
        kill_death = 0.0

    try:
        kills_per_minute = round(kills / (seconds / 60), 2)
        score_per_minute = round(score / (seconds / 60), 2)
    except (ZeroDivisionError, KeyError):
        kills_per_minute = 0.0
        score_per_minute = 0.0

    try:
        kills_per_match = round(kills / matches_played, 2)
    except (ZeroDivisionError, KeyError):
        kills_per_match = 0.0
    except TypeError:
        kills_per_match = 0.0

    return {
        "rank": stats_dict.get("rank", 0),
        "kills": kills,
        "deaths": deaths,
        "wins": wins,
        "loses": losses,
        "killsPerMatch": kills_per_match,
        "headShots": headshot_amount,
        "winPercent": format_percentage_value(win_percent, format_values),
        "headshots": format_percentage_value(headshots, format_values),
        "killDeath": kill_death,
        "timePlayed": str(datetime.timedelta(seconds=stats_dict.get("time", 0))),
        "secondsPlayed": seconds,
        "scorePerMinute": score_per_minute,
        "killsPerMinute": kills_per_minute,
        "longestHeadShot": stats_dict.get("longest_hs", 0),
        "dogtagsTaken": stats_dict.get("dogtags", 0),
        "awardScore": stats_dict.get("awardScore", 0),
        "bonusScore": stats_dict.get("bonusScore", 0),
        "squadScore": stats_dict.get("squadScore", 0),
        "avengerKills": stats_dict.get("avenger_kills", 0),
        "saviorKills": stats_dict.get("savior_kills", 0),
        "shotsFired": shots_fired,
        "shotsHit": shots_hit,
        "accuracy": format_percentage_value(accuracy, format_values),
        "killAssists": stats_dict.get("kill_assists", 0),
        "heals": stats_dict.get("heals", 0),
        "revives": stats_dict.get("revives", 0),
        "repairs": stats_dict.get("repairs", 0),
        "resupplies": stats_dict.get("resupplies", 0),
        "dividedResupplies": {
            "support": stats_dict.get("resupplies_support", 0),
            "medic": stats_dict.get("resupplies_medic", 0),
        },
        "devidedDamage": {
            "toVehicle": stats_dict.get("vehicle_damage", 0),
            "toSoldier": stats_dict.get("soldier_damages", 0),
        },
        "highestKillStreak": stats_dict.get("highest_ks", 0),
        "roundsPlayed": stats_dict.get("rounds", 0),
        "objective": {
            "score": stats_dict.get("sc_objective", 0),
            "armed": stats_dict.get("objective_armed", 0),
            "destroyed": stats_dict.get("objective_destroyed", 0),
            "disarmed": stats_dict.get("objective_disarmed", 0),
        },
        "dividedKills": {
            "nemesis": stats_dict.get("nemesis_kills", 0),
            "pvp": stats_dict.get("kills_pvp", 0),
            "offensive": stats_dict.get("kills_offensive", 0),
            "defensive": stats_dict.get("kills_defensive", 0),
            "aggregated": stats_dict.get("kills_aggregated", 0),
        },
        "devidedScore": {
            "weapons": {
                "general": stats_dict.get("sc_weapons", 0),
                "mmgs": stats_dict.get("sc_mmgs", 0),
                "lmgs": stats_dict.get("sc_lmgs", 0),
                "amrs": stats_dict.get("sc_amr", 0),
                "sniperRifles": stats_dict.get("sc_sniperrifles", 0),
                "assaultRifles": stats_dict.get("sc_assaultrifles", 0),
                "shotguns": stats_dict.get("sc_shotguns", 0),
                "sars": stats_dict.get("sc_sars", 0),
            },
            "vehicles": {
                "general": stats_dict.get("sc_vehicles", 0),
                "cars": stats_dict.get("sc_car", 0),
                "transportVehicles": stats_dict.get("sc_transportvehicles", 0),
                "tanks": stats_dict.get("sc_tanks", 0),
            },
            "awards": stats_dict.get("sc_award", 0),
            "objective": stats_dict.get("sc_objective", 0),
            "gamemode": stats_dict.get("sc_gamemode", 0),
            "combat": stats_dict.get("sc_combat", 0),
            "squad": stats_dict.get("sc_squad", 0),
            "defensive": stats_dict.get("sc_defensive", 0),
            "allAndVehicles": stats_dict.get("sc_alllandvehicles", 0),
        },
    }


async def get_stats(data: dict, format_values: bool = True):
    result = []
    for player in data.get("playerStats", []):
        stats_dict = {}
        for category in player.get("categories", []):
            if category["catName"] == "casablanca_mp_pc":
                for field in category.get("catFields", {"fields": []}).get(
                    "fields", []
                ):
                    stats_dict[field["name"]] = field.get("value", None)
                break

        current_result = {}

        tasks = []
        tasks.append(get_weapons(stats_dict, BF5.WEAPONS, format_values))
        tasks.append(get_vehicles(stats_dict, BF5.VEHICLES))
        tasks.append(get_classes(stats_dict, format_values))
        tasks.append(get_gamemodes(stats_dict, format_values))
        tasks.append(get_maps(stats_dict))

        (
            current_result["weapons"],
            current_result["vehicles"],
            current_result["classes"],
            current_result["gamemodes"],
            current_result["maps"],
        ) = await asyncio.gather(*tasks)

        current_result["id"] = player.get("player", 0)
        current_result.update(await get_main_stats(stats_dict, format_values))
        result.append(current_result)
    if len(result) > 1:
        return {"data": result}
    return result[0]



import asyncio
from typing import Any
import datetime
import logging

from global_mapping import bf6 as BF6
from global_mapping.readability.exceptions import NotFoundException
from global_mapping.readability.shared import format_percentage_value

logger = logging.getLogger("api")
DEFAULT_SEASON = "Season1"


async def serverList(servers):
    for server in servers:
        internal_map_name = server.get("currentMap", "")
        internal_mode_name = server.get("mode", "")
        internal_region_name = server.get("region", "")
        server["regionId"] = internal_region_name
        # compat with bf1:
        try:
            server["ownerId"] = f"{server['owner']['personaId']}"
        except KeyError:
            server["ownerId"] = ""

        try:
            server["currentMap"] = BF6.MAPS[internal_map_name]
            server["url"] = BF6.MAP_PICTURES[internal_map_name]
        except KeyError:
            logger.warning(f"new map, bf2042: {internal_map_name}")
            server["currentMap"] = internal_map_name
            server["url"] = ""
        try:
            server["mode"] = BF6.MODES[internal_mode_name]
            server["smallMode"] = BF6.SMALLMODES[internal_mode_name]
        except KeyError:
            logger.warning(f"new mode, bf2042: {internal_mode_name}")
            server["mode"] = internal_mode_name
            server["smallMode"] = ""
        try:
            server["owner"]["platform"] = BF6.PLATFORM[server["owner"]["platformId"]]
        except KeyError:
            server_owner = server.get("owner")
            logger.warning(f"new platform, bf6: {server_owner.get('platformId', 0)}")
            server["owner"]["platform"] = server_owner.get("platformId", 0)
        try:
            server["region"] = BF6.REGIONS[internal_region_name]
        except KeyError:
            logger.warning(f"new region, bf6: {internal_region_name}")
            server["region"] = ""
    return servers


async def detailedServer(server, lang: str):
    server_info = server.get("server_info", {})

    # if (
    #     current_map is not None
    #     and BF2042.MAP_FACTIONS.get(current_map.get("mapname", "")) is not None
    # ):
    #     teams = BF2042.MAP_FACTIONS.get(
    #         current_map.get("mapname", ""),
    #         [{"name": "team 1", "image": None}, {"name": "team 2", "image": None}],
    #     )
    #     server_info["teams"] = {
    #         "teamOne": BF2042.FACTIONS.get(teams[0], {"name": "team 1", "image": None}),
    #         "teamTwo": BF2042.FACTIONS.get(teams[1], {"name": "team 2", "image": None}),
    #     }
    # else:
    #     server_info["teams"] = {
    #         "teamOne": {"name": "team 1", "image": None},
    #         "teamTwo": {"name": "team 2", "image": None},
    #     }

    # mapnames and modes

    internal_map_name = server_info.get("game", {}).get("gameMap", "")
    internal_mode_name = server_info.get("game", {}).get("gameMode", "")
    try:
        server_info["currentMap"] = BF6.MAPS[internal_map_name]
        server_info["currentMapImage"] = BF6.MAP_PICTURES[internal_map_name]
    except KeyError:
        logger.warning(f"new map, BF6: {internal_map_name}")
        server_info["currentMap"] = internal_map_name
        server_info["currentMapImage"] = ""
    try:
        server_info["mode"] = BF6.MODES[internal_mode_name]
    except KeyError:
        logger.warning(f"new mode, BF6: {internal_mode_name}")
        server_info["mode"] = internal_mode_name

    internal_region_name = server_info.get("pingSite", "")

    # platform name
    if server_info.get("owner", None) is not None:
        server_owner = server_info.get("owner", {})
        server_owner["id"] = server_owner.get("personaId", 0)
        try:
            server_owner["platform"] = BF6.PLATFORM[server_owner["platformId"]]
        except KeyError:
            logger.warning(
                f"new platform, bf2042: {server['server_info']['owner']['platformId']}"
            )
            server_owner["platform"] = server_owner["platformId"]
    if server_info.get("configCreator", None) is not None:
        config_creator = server_info.get("configCreator", {})
        config_creator["id"] = config_creator.get("personaId", 0)
        try:
            config_creator["platform"] = BF6.PLATFORM[config_creator["platformId"]]
        except KeyError:
            logger.warning(
                f"new platform, bf2042: {config_creator.get('platformId', 0)}"
            )
            config_creator["platform"] = config_creator.get("platformId", 0)

    # regionname
    try:
        server_info["region"] = BF6.SHORT_REGIONS[internal_region_name]
    except KeyError:
        logger.warning(f"new region, bf2042: {internal_region_name}")
        server_info["region"] = ""

    # extra_info = server_info.get("serverInfo", {})

    # to make it closer to bf1:
    # server_info["prefix"] = extra_info.get("serverName", "")
    # server_info["description"] = extra_info.get("serverDescription", "")

    # settings_translation = await settingsLanguage(lang)
    # # server settings
    # await settingTranslation(settings_translation, server_info.get("settings", {}))

    # regexp = re.compile(r"^ID_.*_DESC$")
    # if extra_info is not None:
    #     if regexp.search(extra_info.get("configDescription", "")):
    #         extra_info["configNameTranslation"] = settings_translation.get(
    #             extra_info.get("configName", ""), None
    #         )
    #         extra_info["configDescriptionTranslation"] = settings_translation.get(
    #             extra_info.get("configDescription", ""), None
    #         )
    #     else:
    #         extra_info["configNameTranslation"] = ""
    #         extra_info["configDescriptionTranslation"] = ""

    return server


async def get_stats(
    data: dict, category_name: str, format_values: bool = True, multiple: bool = False
):
    result = []
    for player in data.get("playerStats", []):
        current_result = {}
        current_player = player.get("player", {"personaId": 0})
        global_stats = {}
        result_count = 0
        # NEW: We're going to save all the catFields here for Redsec.
        all_fields: list[dict[str, Any]] = []

        for category in player.get("categories", []):
            all_fields.extend(category.get("catFields") or [])
            if category_name == category.get("catName", ""):
                result_count += len(category.get("catFields", []))
                for item in category.get("catFields", []):
                    fields: list[dict[str, str]] = item.get("fields", [])
                    if any(field.get("value", "") == "global" for field in fields):
                        global_stats[item.get("name")] = item.get("value")

        current_result["hasResults"] = result_count > 0
        tasks = []
        filtered_modes = {
            k: v for k, v in BF6.MODES.items() if k not in BF6.REDSEC_MODES.keys()
        }
        tasks.append(get_weapons(global_stats, BF6.WEAPONS, format_values))
        tasks.append(get_vehicles(global_stats, BF6.VEHICLES))
        tasks.append(get_weapons(global_stats, BF6.WEAPON_GROUPS, format_values))
        tasks.append(get_vehicles(global_stats, BF6.VEHICLE_GROUPS))
        tasks.append(get_classes(global_stats))
        tasks.append(get_maps(global_stats, format_values))
        tasks.append(get_gamemodes(global_stats, format_values))
        tasks.append(get_seasons(all_fields, filtered_modes))
        tasks.append(get_gadgets(global_stats, BF6.GADGETS))
        tasks.append(get_gadgets(global_stats, BF6.GADGET_GROUPS))
        tasks.append(get_seasons(all_fields, BF6.REDSEC_MODES))

        if multiple:
            platform_id = current_player.get("platformId", 0)
            current_result["id"] = current_player.get("personaId", 0)
            current_result["userId"] = current_player.get("nucleusId", 0)
            current_result["platform"] = BF6.STATS_PLATFORM.get(
                current_player.get("platformId", 0), "pc"
            )
            current_result["platformId"] = 0 if platform_id is None else platform_id

        (
            current_result["weapons"],
            current_result["vehicles"],
            current_result["weaponGroups"],
            current_result["vehicleGroups"],
            current_result["classes"],
            current_result["maps"],
            current_result["gameModes"],
            current_result["seasons"],
            current_result["gadgets"],
            current_result["gadgetGroups"],
            current_result["redsec"],
        ) = await asyncio.gather(*tasks)

        kit_best_kills = 0
        best_kit = 0
        for kit in current_result["classes"]:
            if kit["kills"] > kit_best_kills:
                kit_best_kills = kit["kills"]
                best_kit = kit["className"]
        current_result["bestClass"] = best_kit

        current_result.update(await get_main_stats(global_stats, format_values))

        result.append(current_result)
    if len(result) < 1:
        raise NotFoundException("player not found")
    if len(result) > 1:
        return {"data": result}
    return result[0]


async def get_weapons(stats_dict: dict, constant: dict, format_values: bool = True):
    weapons = []
    for _id, extra in constant.items():
        kills = stats_dict.get(f"kw_wp_{_id}", 0)
        damage = stats_dict.get(f"dmg_wp_{_id}", 0)
        headshots = stats_dict.get(f"hsw_wp_{_id}", 0)
        shots_hit = stats_dict.get(f"shw_wp_{_id}", 0)
        shots_fired = stats_dict.get(f"sfw_wp_{_id}", 0)
        seconds = stats_dict.get(f"tp_wp_{_id}", 0)

        try:
            accuracy = round((shots_hit / shots_fired) * 100, 2)
        except ZeroDivisionError:
            accuracy = 0.0

        try:
            headshot_percentage = round(headshots / kills * 100, 2)
        except ZeroDivisionError:
            headshot_percentage = 0.0

        try:
            kills_per_minute = round(kills / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            kills_per_minute = 0.0

        try:
            damage_per_minute = round(damage / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            damage_per_minute = 0.0

        try:
            hits_per_kill = round(shots_hit / kills, 2)
        except ZeroDivisionError:
            hits_per_kill = 0.0

        weapons.append(
            {
                **extra,
                "id": _id,
                "kills": kills,
                "damage": damage,
                "assistsDamage": stats_dict.get(f"assdmg_wp_{_id}", 0),
                "bodyKills": stats_dict.get(f"bkw_wp_{_id}", 0),
                "headshotKills": headshots,
                "hipfireKills": stats_dict.get(f"hfkw_wp_{_id}", 0),
                "multiKills": stats_dict.get(f"mkw_wp_{_id}", 0),
                "accuracy": format_percentage_value(accuracy, format_values),
                "killsPerMinute": kills_per_minute,
                "damagePerMinute": damage_per_minute,
                "headshots": format_percentage_value(
                    headshot_percentage, format_values
                ),
                "hitVKills": hits_per_kill,
                "shotsHit": shots_hit,
                "shotsFired": shots_fired,
                "scopedKills": stats_dict.get(f"adskw_wp_{_id}", 0),
                "spawns": stats_dict.get(f"spawns_wp_{_id}", 0),
                "timeEquipped": seconds,
            }
        )
    return weapons


async def get_vehicles(stats_dict: dict, constant: dict):
    vehicles = []
    for _id, extra in constant.items():
        kills = stats_dict.get(f"kw_veh_{_id}", 0)
        seconds = stats_dict.get(f"tp_veh_{_id}", 0)
        try:
            kills_per_minute = round(kills / (seconds / 60), 2)
        except ZeroDivisionError:
            kills_per_minute = 0
        vehicles.append(
            {
                **extra,
                "id": _id,
                "kills": kills,
                "killsPerMinute": kills_per_minute,
                "damage": stats_dict.get(f"dmg_veh_{_id}", 0),
                "spawns": stats_dict.get(f"spawns_veh_{_id}", 0),
                "roadKills": stats_dict.get(f"roadkills_veh_{_id}", 0),
                "passengerAssists": stats_dict.get(f"assists_p_veh_{_id}", 0),
                "multiKills": stats_dict.get(f"mkw_veh_{_id}", 0),
                "distanceTraveled": stats_dict.get(f"disttrv_veh_{_id}", 0),
                "driverAssists": stats_dict.get(f"assists_d_veh_{_id}", 0),
                "vehiclesDestroyedWith": stats_dict.get(f"vdw_veh_{_id}", 0),
                "assists": stats_dict.get(f"assists_veh_{_id}", 0),
                "damageTo": stats_dict.get(f"dmgTo_veh_{_id}", 0),
                "destroyed": stats_dict.get(f"vd_veh_{_id}", 0),
                # "idk": stats_dict.get(f"eor_vh_{_id}"),
                "timeIn": seconds,
            }
        )
    return vehicles


async def get_classes(stats_dict: dict):
    kits = []
    for kit_id, extra in BF6.CLASSES.items():
        kills = stats_dict.get(f"kw_kit_{kit_id}", 0)
        deaths = stats_dict.get(f"deaths_kit_{kit_id}", 0)
        seconds = stats_dict.get(f"tp_kit_{kit_id}", 0)
        try:
            kill_death = round(kills / deaths, 2)
        except ZeroDivisionError:
            kill_death = 0.0
        try:
            kills_per_minute = round(kills / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            kills_per_minute = 0.0
        kits.append(
            {
                **extra,
                "id": kit_id,
                "kills": kills,
                "deaths": deaths,
                "kpm": kills_per_minute,
                "killDeath": kill_death,
                "spawns": stats_dict.get(f"spawns_kit_{kit_id}", 0),
                "score": stats_dict.get(f"scoreas_kit_{kit_id}", 0),
                "assists": stats_dict.get(f"assists_kit_{kit_id}", 0),
                "secondsPlayed": seconds,
            }
        )
    return kits


async def get_gadgets(stats_dict: dict, constant: dict):
    gadgets = []
    for _id, extra in constant.items():
        damage = stats_dict.get(f"dmg_gad_{_id}", 0)
        kills = stats_dict.get(f"kw_gad_{_id}", 0)
        seconds = stats_dict.get(f"tp_gad_{_id}", 0)

        try:
            kills_per_minute = round(kills / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            kills_per_minute = 0.0

        try:
            damage_per_minute = round(damage / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            damage_per_minute = 0.0

        gadgets.append(
            {
                **extra,
                "id": _id,
                "kills": kills,
                "assistsDamage": stats_dict.get(f"assdmg_gad_{_id}", 0),
                "assists": stats_dict.get(f"assists_gad_{_id}", 0),
                "spotAssists": stats_dict.get(f"sptass_gad_{_id}", 0),
                "spots": stats_dict.get(f"spot_gad_{_id}", 0),
                "spawns": stats_dict.get(f"spawns3_gad_{_id}", 0),
                # "spawns": stats_dict.get(f"spawns2_gad_{_id}", 0), ????
                "damage": damage,
                "uses": stats_dict.get(f"uses_gad_{_id}", 0),
                "multiKills": stats_dict.get(f"mkw_gad_{_id}", 0),
                "vehiclesDestroyedWith": stats_dict.get(f"vehd_gad_{_id}", 0),
                "kpm": kills_per_minute,
                "dpm": damage_per_minute,
                "secondsPlayed": seconds,
            }
        )
    return gadgets


async def get_maps(stats_dict: dict, format_values: bool = False):
    maps = []
    for _id, extra in BF6.STAT_MAPS.items():
        wins = stats_dict.get(f"wins_lvl{_id}", 0)
        losses = stats_dict.get(f"losses_lvl{_id}", 0)
        try:
            win_percent = round(wins / (wins + losses) * 100, 2)
        except ZeroDivisionError:
            win_percent = 0.0

        maps.append(
            {
                **extra,
                "id": _id,
                "wins": wins,
                "losses": losses,
                "matches": stats_dict.get(f"matches_lvl{_id}", 0),
                "winPercent": format_percentage_value(win_percent, format_values),
                "secondsPlayed": stats_dict.get(f"tp_lvl{_id}", 0),
            }
        )
    return maps


async def get_gamemodes(stats_dict: dict, format_values: bool = False):
    gamemodes = []
    for _id, extra in BF6.STAT_GAMEMODE_SMALL.items():
        wins = stats_dict.get(f"wins_gm_{_id}", 0)
        losses = stats_dict.get(f"losses_gm_{_id}", 0)
        damage = stats_dict.get(f"dmg_gm_{_id}", 0)
        kills = stats_dict.get(f"kills_gm_{_id}", 0)
        deaths = stats_dict.get(f"deaths_gm_{_id}", 0)
        seconds = stats_dict.get(f"tp_gm_{_id}", 0)
        headshots = stats_dict.get(f"hsw_gm_{_id}", 0)

        try:
            win_percent = round(wins / (wins + losses) * 100, 2)
        except ZeroDivisionError:
            win_percent = 0.0

        try:
            kills_per_minute = round(kills / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            kills_per_minute = 0.0

        try:
            damage_per_minute = round(damage / (seconds / 60), 2)
        except (ZeroDivisionError, KeyError):
            damage_per_minute = 0.0

        try:
            kill_death = round(kills / deaths, 2)
        except ZeroDivisionError:
            kill_death = 0.0

        try:
            headshot_percentage = round(headshots / kills * 100, 2)
        except ZeroDivisionError:
            headshot_percentage = 0.0

        gamemodes.append(
            {
                **extra,
                "id": _id,
                "kills": kills,
                "deaths": deaths,
                "wins": wins,
                "losses": losses,
                "killDeath": kill_death,
                "winPercent": format_percentage_value(win_percent, format_values),
                "killAssists": stats_dict.get(f"assists_gm_{_id}", 0),
                "matches": stats_dict.get(f"matches_gm_{_id}", 0),
                "repairs": stats_dict.get(f"repair_gm_{_id}", 0),
                "revives": stats_dict.get(f"revives_gm_{_id}", 0),
                "spots": stats_dict.get(f"spot_gm_{_id}", 0),
                "objectiveTime": stats_dict.get(f"obj_time_gm_{_id}", 0),
                "objectivesCaptured": stats_dict.get(f"obj_captured_gm_{_id}", 0),
                "objectivesDefended": stats_dict.get(f"obj_defended_gm_{_id}", 0),
                "scoreIn": stats_dict.get(f"scorein_gm_{_id}", 0),
                "headshotKills": headshots,
                "headshots": format_percentage_value(
                    headshot_percentage, format_values
                ),
                "kpm": kills_per_minute,
                "dpm": damage_per_minute,
                "secondsPlayed": seconds,
            }
        )
    return gamemodes


async def get_main_stats(stats_dict: dict, format_values: bool = True):
    shots_fired = stats_dict.get("shots_fired_total", 0)
    shots_hit = stats_dict.get("shots_hit_total", 0)
    wins = stats_dict.get("won_match_total", 0)
    losses = stats_dict.get("lost_match_total", 0)
    kills = stats_dict.get("Kills_Total", 0)
    deaths = stats_dict.get("deaths_total", 0)
    seconds = stats_dict.get("tp_gm_all", 0)
    damage = stats_dict.get("dmg_total", 0)
    human_kills = stats_dict.get("human_kills_total", 0)
    headshot_amount = stats_dict.get("kills_Headshots_Total", 0)
    matches_played = stats_dict.get("played_match_total", 0)
    try:
        accuracy = round((shots_hit / shots_fired) * 100, 2)
    except ZeroDivisionError:
        accuracy = 0.0
    try:
        win_percent = round(wins / (wins + losses) * 100, 2)
    except ZeroDivisionError:
        win_percent = 0.0
    try:
        headshots = round((headshot_amount / kills) * 100, 2)
    except ZeroDivisionError:
        headshots = 0.0
    try:
        kill_death = round(kills / deaths, 2)
        infantry_kill_death = round(human_kills / deaths, 2)
    except ZeroDivisionError:
        kill_death = 0.0
        infantry_kill_death = 0.0
    try:
        kills_per_minute = round(kills / (seconds / 60), 2)
        damage_per_minute = round(damage / (seconds / 60), 2)
    except (ZeroDivisionError, KeyError):
        kills_per_minute = 0.0
        damage_per_minute = 0.0

    try:
        kills_per_match = round(kills / matches_played, 2)
        damage_per_match = round(damage / matches_played, 2)
    except (ZeroDivisionError, KeyError):
        kills_per_match = 0.0
        damage_per_match = 0.0

    try:
        human_precentage = round((human_kills / kills) * 100, 2)
    except ZeroDivisionError:
        human_precentage = 0.0

    return {
        "humanPrecentage": format_percentage_value(human_precentage, format_values),
        "kills": kills,
        "deaths": deaths,
        "wins": wins,
        "loses": losses,
        "killsPerMinute": kills_per_minute,
        "damagePerMinute": damage_per_minute,
        "killsPerMatch": kills_per_match,
        "damagePerMatch": damage_per_match,
        "headShots": headshot_amount,
        "winPercent": format_percentage_value(win_percent, format_values),
        "headshots": format_percentage_value(headshots, format_values),
        "killDeath": kill_death,
        "infantryKillDeath": infantry_kill_death,
        "damage": damage,
        "timePlayed": str(datetime.timedelta(seconds=stats_dict.get("tp_gm_all", 0))),
        "accuracy": format_percentage_value(accuracy, format_values),
        "revives": stats_dict.get("Revives_Teammates_Total", 0),
        "heals": stats_dict.get("Dmg_Healed_Total", 0),
        "resupplies": stats_dict.get("Resupply_Teammates_Total", 0),
        "repairs": stats_dict.get("Veh_RepairedHP_Total", 0),
        "squadmateRevive": stats_dict.get("Revives_Squadmates_Total", 0),
        "thrownThrowables": stats_dict.get("thrown_throwables_total", 0),
        "inRound": {
            "revives": stats_dict.get("revives_inround_cb", 0),
            "resupplies": stats_dict.get("resupply_teammates_inround_cb", 0),
            "spotAssists": stats_dict.get("spot_assist_inround_cb", 0),
            "thrownThrowables": stats_dict.get("thrown_throwables_inround_cb", 0),
            "playerTakeDowns": stats_dict.get("players_taken_down_inround_cb", 0),
        },
        "gadgetsDestoyed": stats_dict.get("destroy_gadget_total", 0),
        "playerTakeDowns": (stats_dict.get("Players_Taken_Down_Total", 0)),
        "matchesPlayed": matches_played,
        "secondsPlayed": seconds,
        "dividedSecondsPlayed": {
            "flying": stats_dict.get("flying_time_total", 0),
            "driving": stats_dict.get("driving_time_total", 0),
        },
        "saviorKills": stats_dict.get("savior_kills_total", 0),
        "shotsFired": shots_fired,
        "shotsHit": shots_hit,
        "killAssists": stats_dict.get("assists_gm_all", 0),
        "vehiclesDestroyed": stats_dict.get("Destroyed_Veh_Total", 0),
        "enemiesSpotted": stats_dict.get("Spotted_Enemies_Total", 0),
        "dividedKills": {
            "ads": stats_dict.get("Kills_ADS_Total", 0),
            "grenades": stats_dict.get("Kills_Grenade_Total", 0),
            "hipfire": stats_dict.get("Kills_Hipfire_Total", 0),
            "longDistance": stats_dict.get("Kills_LongDist_Total", 0),
            "melee": stats_dict.get("Kills_Melee_Total", 0),
            "multiKills": stats_dict.get("Kills_Multi_Total", 0),
            "passenger": stats_dict.get("Kills_Psgr_Total", 0),
            "vehicle": stats_dict.get("Kills_wVeh_Total", 0),
            "roadkills": stats_dict.get("kill_road_total", 0),
            "human": human_kills,
            "weapons": {
                "SMG": stats_dict.get("kills_smg_total", 0),
                "LMG": stats_dict.get("kills_lmg_total", 0),
                "DMR": stats_dict.get("kills_dmr_total", 0),
                "Shotguns": stats_dict.get("kills_sg_total", 0),
                "Assault Rifles": stats_dict.get("kills_sg_total", 0),
            },
            "inRound": {
                "total": stats_dict.get("kills_inround_cb", 0),
                "grenade": stats_dict.get("kills_grenade_inround_cb", 0),
                "headshots": stats_dict.get("kills_headshots_inround_cb", 0),
                "melee": stats_dict.get("kills_melee_inround_cb", 0),
                "multiKills": stats_dict.get("kills_multi_inround_cb", 0),
                "weapons": {
                    "SMG": stats_dict.get("kills_smg_inround_cb", 0),
                    "AR": stats_dict.get("kills_ar_inround_cb", 0),
                    "DMR": stats_dict.get("kills_dmr_inround_cb", 0),
                },
            },
        },
        "devidedDamage": {
            "passenger": stats_dict.get("Dmg_DealtPsgr_Total", 0),
            "vehicleDriver": stats_dict.get("dmg_vehdriver_total", 0),
            "toVehicle": stats_dict.get("Dmg_Dealt_To_Veh_Total", 0),
            "inRound": {
                "asVehicle": stats_dict.get("dmgwith_veh_inround_cb", 0),
            },
        },
        "devidedAssists": {
            "driver": stats_dict.get("Assists_Driver_Total", 0),
            "passenger": stats_dict.get("Assists_Psgr_Total", 0),
            "spot": stats_dict.get("Assists_Spot_Total", 0),
            "pilot": stats_dict.get("assists_pilot_total", 0),
            "inRound": {
                "total": stats_dict.get("assists_inround_cb", 0),
            },
        },
        "distanceTraveled": {
            "foot": stats_dict.get("distrav_foot_total", 0),
            "passenger": stats_dict.get("distrav_psgr_total", 0),
            "vehicle": stats_dict.get("distrav_veh_total", 0),
        },
        "sector": {
            "captured": stats_dict.get("sect_cap_total", 0),
        },
        "objective": {
            "time": {
                "total": stats_dict.get("Obj_Time_Total", 0),
                "attacked": stats_dict.get("obj_attack_time_Total", 0),
                "defended": stats_dict.get("Obj_DefendTime_Total", 0),
            },
            "armed": stats_dict.get("obj_arm_total", 0),
            "captured": stats_dict.get("Obj_Captured_Total", 0),
            "neutralized": stats_dict.get("obj_neutralized_total", 0),
            "defused": stats_dict.get("obj_defuse_total", 0),
            "destroyed": stats_dict.get("Obj_Destroyed_Total", 0),
            "inRound": {
                "armed": stats_dict.get("obj_arm_inround_cb", 0),
                "captured": stats_dict.get("cap_obj_inround_cb", 0),
                "neutralized": stats_dict.get("cap_neut_obj_inround_cb", 0),
                "defused": stats_dict.get("obj_defuse_inround_cb", 0),
                "destroyed": stats_dict.get("obj_dest_inround_cb", 0),
            },
        },
        "XP": (
            {
                "total": stats_dict.get("xp_all", 0),
                "performance": stats_dict.get("xp_performance", 0),
                "accolades": stats_dict.get("xp_accolades", 0),
            },
        ),
    }


async def players(player_list):
    for player in player_list.get("results", []):
        player["platformId"] = player.get("platform", 0)
        player["platform"] = BF6.STATS_PLATFORM.get(player.get("platform", 0), "pc")
    return player_list


async def get_seasons(all_fields: list[dict[str, Any]], MODES: dict):
    seasons = []
    for season_id, season in BF6.SEASONS.items():
        modes = []
        for internal_mode_name, mode_name in MODES.items():
            index: dict[str, int] = {}
            for item in all_fields:
                fields = item.get("fields", [])
                gm = None
                sn = None

                for f in fields:
                    fname = f.get("name")
                    if fname == "GameMode":
                        gm = f.get("value")
                    elif fname == "Season":
                        sn = f.get("value")

                if gm == internal_mode_name and sn == season_id:
                    index[item.get("name", "")] = item.get("value", 0)

            if not index:
                continue

            def first(names: list[str]) -> int:
                for n in names:
                    if n in index:
                        return index[n]
                return 0

            played = first(
                [
                    "played_match_total",
                    "matches_lvlftpgranite",
                    "matches_lvlftp",
                    "matches_level",
                ]
            )

            wins = first(
                [
                    "won_match_total",
                    "wins_lvlftpgranite",
                    "wins_lvlftp",
                    "wins_level",
                ]
            )

            losses = first(
                [
                    "lost_match_total",
                    "losses_lvlftpgranite",
                    "losses_lvlftp",
                    "losses_level",
                ]
            )

            kills = first(["human_kills_total"])
            deaths = first(["deaths_total"])
            score = first(["score_total"])
            time_played = first(["tp_level"])  # seconds
            extractions = first(["extract_extracted_total"])

            kd = round(kills / deaths, 2) if deaths else None
            # winrate = round((wins / played) * 100, 2) if played else None

            modes.append(
                {
                    "modeId": internal_mode_name,
                    "mode": mode_name,
                    "matches": played,
                    "wins": wins,
                    "losses": losses,
                    "kills": kills,
                    "deaths": deaths,
                    "killDeath": kd,
                    "score": score,
                    "secondsPlayed": time_played,
                    "timePlayed": str(datetime.timedelta(seconds=time_played)),
                    "extractions": extractions,
                }
            )

        seasons.append(
            {
                "seasonId": season_id,
                "season": season,
                "modes": modes,
            }
        )
    return seasons



SEASONS = {"Season1": "Season 1"}
PLATFORM = {
    0: "unknown",
    1: "pc",
    2: "ps4",
    3: "xboxone",
    4: "ps5",
    5: "xboxseries",
    6: "common",
    7: "steam",
}
STATS_PLATFORM = {
    1: "pc",
    3: "xboxone",
    2: "ps4",
    5: "xboxseries",
    4: "ps5",
    6: "common",
    7: "steam",
}
PLATFORM_REVERSE = {
    "unknown": 0,
    "pc": 1,
    "ps4": 2,
    "xboxone": 3,
    "ps5": 4,
    "xboxseries": 5,
    "common": 6,
    "steam": 7,
}
PLATFORM_FESL = {
    0: "pc",
    1: "pc",
    2: "ps4",
    3: "xboxone",
    4: "ps4",
    5: "xboxone",
    6: "pc",
    7: "pc",
}
STATS_PLATFORM_REVERSE = {
    "pc": 1,
    "xboxone": 3,
    "ps4": 2,
    "xboxseries": 5,
    "ps5": 4,
    "xbox": 5,
    "psn": 4,
    "steam": 7,
}
REGIONS = {
    "aws-hkg": "Asia",
    "aws-icn": "Asia",
    "aws-nrt": "Asia",
    "aws-sin": "Asia",
    "aws-iad": "North America",
    "aws-pdx": "North America",
    "aws-sjc": "North America",
    "aws-brz": "South America",
    "aws-fra": "Europe",
    "aws-lhr": "Europe",
    "aws-syd": "Oceania",
}
REGIONSLIST = {
    "asia": ["aws-hkg", "aws-icn", "aws-nrt", "aws-sin"],
    "nam": ["aws-iad", "aws-pdx", "aws-sjc"],
    "sam": ["aws-brz"],
    "eu": ["aws-fra", "aws-lhr"],
    "oc": ["aws-syd"],
    "all": [
        "aws-iad",
        "aws-pdx",
        "aws-sjc",
        "aws-brz",
        "aws-fra",
        "aws-lhr",
        "aws-hkg",
        "aws-icn",
        "aws-nrt",
        "aws-sin",
        "aws-syd",
    ],
}
SHORT_REGIONS = {
    "aws-hkg": "asia",
    "aws-icn": "asia",
    "aws-nrt": "asia",
    "aws-sin": "asia",
    "aws-iad": "nam",
    "aws-pdx": "nam",
    "aws-sjc": "nam",
    "aws-brz": "sam",
    "aws-fra": "eu",
    "aws-lhr": "eu",
    "aws-syd": "oc",
}
MAPS = {
    "MP_Abbasid": "Siege of Cairo",
    "MP_Aftermath": "Empire State",
    "MP_Badlands": "Blackwell Fields",
    "MP_Battery": "Iberian Offensive",
    "MP_Capstone": "Liberation Peak",
    "MP_Dumbo": "Manhattan Bridge",
    "MP_Eastwood": "Eastwood",
    "MP_FireStorm": "Operation Firestorm",
    "MP_Limestone": "Saints Quarter",
    "MP_Outskirts": "New Sobek City",
    "MP_Tungsten": "Mirak Valley",
    "MP_Portal_Sand": "Empire State",
    "MP_Granite": "Fort Lyndon",
    "MP_Granite_MainStreet_Portal": "Downtown",
    # "MP_Granite_MilitaryRnD_Portal": "",
    "MP_Granite_Marina_Portal": "Eastwood",
    "MP_Granite_ClubHouse_Portal": "Club House",
    "MP_Granite_TechCampus_Portal": "Tech Center",
}
TO_GAME_MAPS = {
    "siege of cairo": "MP_Abbasid",
    "empire state": "MP_Aftermath",
    "blackwell fields": "MP_Badlands",
    "iberian offensive": "MP_Battery",
    "liberation peak": "MP_Capstone",
    "manhattan bridge": "MP_Dumbo",
    "eastwood": "MP_Eastwood",
    "operation firestorm": "MP_FireStorm",
    "saints quarter": "MP_Limestone",
    "new sobek city": "MP_Outskirts",
    "mirak valley": "MP_Tungsten",
    # "empire state": "MP_Portal_Sand",
    "downtown": "MP_Granite_MainStreet_Portal",
    "club house": "MP_Granite_ClubHouse_Portal",
    "tech center": "MP_Granite_TechCampus_Portal",
}
MAP_TRANSLATION_IDS = {
    "MP_Abbasid": "ID_MP_LVL_ABBASID_NAME",
    "MP_Aftermath": "ID_MP_LVL_AFTERMATH_NAME",
    "MP_Badlands": "ID_MP_LVL_BADLANDS_NAME",
    "MP_Battery": "ID_MP_LVL_BATTERY_NAME",
    "MP_Capstone": "ID_MP_LVL_CAPSTONE_NAME",
    "MP_Dumbo": "ID_MP_LVL_DUMBO_NAME",
    "MP_Eastwood": "ID_MP_LVL_EASTWOOD_NAME",
    "MP_FireStorm": "ID_MP_LVL_FIRESTORM_NAME",
    "MP_Limestone": "ID_MP_LVL_LIMESTONE_NAME",
    "MP_Outskirts": "ID_MP_LVL_OUTSKIRTS_NAME",
    "MP_Tungsten": "ID_MP_LVL_TUNGSTEN_NAME",
    "MP_Portal_Sand": "ID_MP_LVL_AFTERMATH_NAME",
    "MP_Granite": "ID_ARRIVAL_MAP_GRANITE",
    "MP_Granite_MainStreet_Portal": "ID_ARRIVAL_MAP_GRANITE_DOWNTOWN",
    "MP_Granite_Marina_Portal": "ID_MP_LVL_EASTWOOD_NAME",
    "MP_Granite_ClubHouse_Portal": "ID_ARRIVAL_MAP_GRANITE_CLUB_HOUSE",
    "MP_Granite_TechCampus_Portal": "ID_ARRIVAL_MAP_GRANITE_TECH_CENTER",
}
MAP_PICTURES = {
    "MP_Abbasid": "https://cdn.gametools.network/maps/bf6/T_UI_Abbasid_Large_OPT-49a3761a.webp",
    "MP_Aftermath": "https://cdn.gametools.network/maps/bf6/T_UI_Aftermath_Large_OPT-bf883df1.webp",
    "MP_Badlands": "https://cdn.gametools.network/maps/bf6/T_UI_Badlands_Large_OPT-6885eb83.webp",
    "MP_Battery": "https://cdn.gametools.network/maps/bf6/T_UI_Battery_Large_OPT-034d4636.webp",
    "MP_Capstone": "https://cdn.gametools.network/maps/bf6/T_UI_Capstone_Large_OPT-2ccae694.webp",
    "MP_Eastwood": "https://cdn.gametools.network/maps/bf6/T_UI_Eastwood_Large_OPT-3bfa6a6f.webp",
    "MP_Dumbo": "https://cdn.gametools.network/maps/bf6/T_UI_Dumbo_Large_OPT-20de031f.webp",
    "MP_FireStorm": "https://cdn.gametools.network/maps/bf6/T_UI_Firestorm_Large_OPT-45d582ad.webp",
    "MP_Limestone": "https://cdn.gametools.network/maps/bf6/T_UI_Limestone_Large_OPT-c9160897.webp",
    "MP_Outskirts": "https://cdn.gametools.network/maps/bf6/T_UI_Outskirts_Large_OPT-bf08f756.webp",
    "MP_Tungsten": "https://cdn.gametools.network/maps/bf6/T_UI_Tungsten_Large_OPT-935da06b.webp",
    "MP_Portal_Sand": "https://cdn.gametools.network/maps/bf6/T_UI_Aftermath_Large_OPT-bf883df1.webp",
    "MP_Granite": "https://cdn.gametools.network/maps/bf6/Fort_Lyndon_BF6_REDSEC.webp",
    "MP_Granite_MainStreet_Portal": "https://cdn.gametools.network/maps/bf6/T_UI_Granite_Portal_01_Thumb_SML-c1924a54.jpg",
    "MP_Granite_Marina_Portal": "https://cdn.gametools.network/maps/bf6/T_UI_Eastwood_Large_OPT-3bfa6a6f.webp",
    "MP_Granite_ClubHouse_Portal": "https://cdn.gametools.network/maps/bf6/T_UI_Granite_Portal_05_Thumb_SML-74ea7be7.webp",
    "MP_Granite_TechCampus_Portal": "https://cdn.gametools.network/maps/bf6/T_UI_Granite_Portal_06_Thumb_SML-80099ba6.webp",
}
SMALLMODES = {
    "Breakthrough0": "BT",
    "BreakthroughSmall0": "BS",
    "ConquestSmall0": "CQ",
    "ModBuilderCustom0": "CM",
    "Rush0": "RS",
    "Conquest0": "CL",
    "GraniteDuo0": "RD",
    "GraniteSquad0": "RQ",
    "GraniteGauntlet0": "RG",
}
MODES = {
    "Breakthrough0": "Breakthrough Large",
    "BreakthroughSmall0": "Breakthrough",
    "ConquestSmall0": "Conquest",
    "ModBuilderCustom0": "Custom",
    "Rush0": "Rush",
    "Conquest0": "Conquest large",
    "GraniteDuo0": "Redsec Duos",
    "GraniteSquad0": "Redsec Quads",
    "GraniteGauntlet0": "Redsec Gauntlet",
}
REDSEC_MODES = {
    "GraniteDuo0": "Duos",
    "GraniteSquad0": "Quads",
    "GraniteGauntlet0": "Gauntlet",
}
TO_GAME_MODES = {
    "breakthrough large": "Breakthrough0",
    "breakthrough": "BreakthroughSmall0",
    "conquest": "ConquestSmall0",
    "custom": "ModBuilderCustom0",
    "rush": "Rush0",
    "conquest large": "Conquest0",
    "redsec duos": "GraniteDuo0",
    "redsec quads": "GraniteSquad0",
    "redsec gauntlet": "GraniteGauntlet0",
}
STAT_MAPS = {
    "mpabbasid": {
        "mapName": "Siege of Cairo",
        "image": "https://cdn.gametools.network/maps/bf6/T_UI_Abbasid_Large_OPT-49a3761a.webp",
    },
    "mpaftermath": {
        "mapName": "Empire State",
        "image": "https://cdn.gametools.network/maps/bf6/T_UI_Aftermath_Large_OPT-bf883df1.webp",
    },
    "mpbattery": {
        "mapName": "Iberian Offensive",
        "image": "https://cdn.gametools.network/maps/bf6/T_UI_Battery_Large_OPT-034d4636.webp",
    },
    "mpcapstone": {
        "mapName": "Liberation Peak",
        "image": "https://cdn.gametools.network/maps/bf6/T_UI_Capstone_Large_OPT-2ccae694.webp",
    },
    "mpdumbo": {
        "mapName": "Manhattan Bridge",
        "image": "https://cdn.gametools.network/maps/bf6/T_UI_Dumbo_Large_OPT-20de031f.webp",
    },
    "mpfirestorm": {
        "mapName": "Operation Firestorm",
        "image": "https://cdn.gametools.network/maps/bf6/T_UI_Firestorm_Large_OPT-45d582ad.webp",
    },
    "mplimestone": {
        "mapName": "Saints Quarter",
        "image": "https://cdn.gametools.network/maps/bf6/T_UI_Limestone_Large_OPT-c9160897.webp",
    },
    "mpoutskirts": {
        "mapName": "New Sobek City",
        "image": "https://cdn.gametools.network/maps/bf6/T_UI_Outskirts_Large_OPT-bf08f756.webp",
    },
    "mptungsten": {
        "mapName": "Mirak Valley",
        "image": "https://cdn.gametools.network/maps/bf6/T_UI_Tungsten_Large_OPT-935da06b.webp",
    },
    "mpbadlands": {
        "mapName": "Blackwell Fields",
        "image": "https://cdn.gametools.network/maps/bf6/Battlefield_6_Blackwell_Fields.webp",
    },
    "ftpgranite": {
        "mapName": "REDSEC",
        "image": "https://cdn.gametools.network/maps/bf6/Battlefield_6_Redsec.webp",
    },
    "mpeastwood": {
        "mapName": "Eastwood",
        "image": "https://cdn.gametools.network/maps/bf6/T_UI_Eastwood_Large_OPT-3bfa6a6f.webp",
    },
}
STAT_GAMEMODE_SMALL = {
    "esc": {"gamemodeName": "Escalation", "image": ""},
    "official": {"gamemodeName": "Official", "image": ""},
    "tdm": {"gamemodeName": "Team deathmatch", "image": ""},
    "cq": {"gamemodeName": "Conquest", "image": ""},
    "granite": {"gamemodeName": "Redsec Squad", "image": ""},
    "graniteDuo": {"gamemodeName": "Redsec Duo", "image": ""},
    "modbuilder0": {"gamemodeName": "Portal", "image": ""},
    "rush": {"gamemodeName": "Rush", "image": ""},
}
STAT_GAMEMODE = {
    "MP_Escalation0": {"gamemodeName": "Escalation", "image": ""},
    "MP_TeamDM0": {"gamemodeName": "Team deathmatch", "image": ""},
    "Conquest0": {"gamemodeName": "Conquest", "image": ""},
    "MP_KOTH0": {"gamemodeName": "King of the hill", "image": ""},
    "MP_SquadDM0": {"gamemodeName": "Squad deathmatch", "image": ""},
    "Breakthrough0": {"gamemodeName": "Breakthrough", "image": ""},
    "Rush0": {"gamemodeName": "Rush", "image": ""},
}
CLASSES = {
    "assault": {
        "className": "Assault",
        "image": "https://cdn.gametools.network/classes/bf6/white/Assault.svg",
        "altImage": "https://cdn.gametools.network/classes/bf6/black/Assault.svg",
    },
    "engineer": {
        "className": "Engineer",
        "image": "https://cdn.gametools.network/classes/bf6/white/Engineer.svg",
        "altImage": "https://cdn.gametools.network/classes/bf6/black/Engineer.svg",
    },
    "support": {
        "className": "Support",
        "image": "https://cdn.gametools.network/classes/bf6/white/Support.svg",
        "altImage": "https://cdn.gametools.network/classes/bf6/black/Support.svg",
    },
    "recon": {
        "className": "Recon",
        "image": "https://cdn.gametools.network/classes/bf6/white/Recon.svg",
        "altImage": "https://cdn.gametools.network/classes/bf6/black/Recon.svg",
    },
}
VEHICLES = {
    "air_panthera": {
        "type": "Air Combat",
        "vehicleName": "Panthera KHT",
        "image": "https://cdn.gametools.network/vehicles/bf6/T_UI_MDV_Eurocopter_VSD0001-8003028d.webp",
        "altImage": "https://cdn.gametools.network/vehicles/bf6/white/T_UI_MDV_Eurocopter_VSD0001-8003028d.webp",
    },
    "air_m77efalchio": {
        "type": "Air Combat",
        "vehicleName": "M77E Falchion",
        "image": "https://cdn.gametools.network/vehicles/bf6/T_UI_MDV_AH64E_VSD0001-dd0a7df6.webp",
        "altImage": "https://cdn.gametools.network/vehicles/bf6/white/T_UI_MDV_AH64E_VSD0001-dd0a7df6.webp",
    },
    "sur_leoa4": {
        "type": "Ground Combat",
        "vehicleName": "Leo A4",
        "image": "https://cdn.gametools.network/vehicles/bf6/T_UI_MDV_Leopard_VSD0001-f8da51ee.webp",
        "altImage": "https://cdn.gametools.network/vehicles/bf6/white/T_UI_MDV_Leopard_VSD0001-f8da51ee.webp",
    },
    "sur_strf09a4": {
        "type": "Ground Combat",
        "vehicleName": "Strf 09 A4",
        "image": "https://cdn.gametools.network/vehicles/bf6/T_UI_CV90_VSD0001-acd942b6.webp",
        "altImage": "https://cdn.gametools.network/vehicles/bf6/white/T_UI_CV90_VSD0001-acd942b6.webp",
    },
    "sur_m1a2sepv3": {
        "type": "Ground Combat",
        "vehicleName": "M1A2 SEPv3",
        "image": "https://cdn.gametools.network/vehicles/bf6/T_UI_MDV_Abrams_VSD0001-5412a78d.webp",
        "altImage": "https://cdn.gametools.network/vehicles/bf6/white/T_UI_MDV_Abrams_VSD0001-5412a78d.webp",
    },
    "sur_cheetah1a2": {
        "type": "Ground Combat",
        "vehicleName": "Cheetah 1A2",
        "image": "https://cdn.gametools.network/vehicles/bf6/T_UI_MDV_Gepard_VSD0001-d796732f.webp",
        "altImage": "https://cdn.gametools.network/vehicles/bf6/white/T_UI_MDV_Gepard_VSD0001-d796732f.webp",
    },
    "sur_glider96": {
        "type": "Ground Combat",
        "vehicleName": "Glider 96",
        "image": "https://cdn.gametools.network/vehicles/bf6/T_UI_MDV_Flyer60_VSD0005-1569869f.webp",
        "altImage": "https://cdn.gametools.network/vehicles/bf6/white/T_UI_MDV_Flyer60_VSD0005-1569869f.webp",
    },
    "sur_bradley": {
        "type": "Ground Combat",
        "vehicleName": "M3A3 Bradley",
        "image": "https://cdn.gametools.network/vehicles/bf6/T_UI_OB_VEH_Tank_Bradley_VSD0001_Dressing-66f252ca.webp",
        "altImage": "https://cdn.gametools.network/vehicles/bf6/white/T_UI_OB_VEH_Tank_Bradley_VSD0001_Dressing-66f252ca.webp",
    },
    "air_su57": {
        "type": "Air Combat",
        "vehicleName": "Su-57",
        "image": "https://cdn.gametools.network/vehicles/bf6/T_UI_MDV_SU57_VSD0001-1b5aa5ee.webp",
        "altImage": "https://cdn.gametools.network/vehicles/bf6/white/T_UI_MDV_SU57_VSD0001-1b5aa5ee.webp",
    },
    "air_f61v": {
        "type": "Air Combat",
        "vehicleName": "F-61V",
        "image": "https://cdn.gametools.network/vehicles/bf6/T_UI_MDV_F16_VSD0001-5f951ec9.webp",
        "altImage": "https://cdn.gametools.network/vehicles/bf6/white/T_UI_MDV_F16_VSD0001-5f951ec9.webp",
    },
    "air_f39e": {
        "type": "Air Combat",
        "vehicleName": "F-39E",
        "image": "https://cdn.gametools.network/vehicles/bf6/T_UI_MDV_JAS39_VSD0001-9313d717.webp",
        "altImage": "https://cdn.gametools.network/vehicles/bf6/white/T_UI_MDV_JAS39_VSD0001-9313d717.webp",
    },
    "air_m77efalchion": {
        "type": "Air Combat",
        "vehicleName": "M77E Falchion",
        "image": "https://cdn.gametools.network/vehicles/bf6/T_UI_MDV_AH64E_VSD0001-dd0a7df6.webp",
        "altImage": "https://cdn.gametools.network/vehicles/bf6/white/T_UI_MDV_AH64E_VSD0001-dd0a7df6.webp",
    },
    "sur_vector": {
        "type": "Ground Transport",
        "vehicleName": "VECTOR",
        "image": "https://cdn.gametools.network/vehicles/bf6/T_UI_MDV_Vector_VSD0002-241efa80.webp",
        "altImage": "https://cdn.gametools.network/vehicles/bf6/white/T_UI_MDV_Vector_VSD0002-241efa80.webp",
    },
    "sur_travmark2": {
        "type": "Ground Transport",
        "vehicleName": "Traverser Mark 2",
        "image": "https://cdn.gametools.network/vehicles/bf6/T_UI_Marauder_VSD0001_Icon_25667539-c29e2338.png",
        "altImage": "https://cdn.gametools.network/vehicles/bf6/white/T_UI_Marauder_VSD0001_Icon_25667539-c29e2338.png",
    },
    "air_uh06": {"type": "Air Transport", "vehicleName": "UH-06", "image": ""},
    "air_mh47": {"type": "Air Transport", "vehicleName": "MH47 Chinook", "image": ""},
    "air_f97kes": {"type": "Air Combat", "vehicleName": "F-97 Kestrel", "image": ""},
    "sur_rhib": {
        "type": "Ground Transport",
        "vehicleName": "7.7m NSW RBHIB",
        "image": "",
    },
    "sur_ptv": {
        "type": "Ground Transport",
        "vehicleName": "Turfpro PTV Royal",
        "image": "",
    },
    "sur_quadbike": {
        "type": "Ground Transport",
        "vehicleName": "Rugged MV740",
        "image": "",
    },
}
VEHICLE_GROUPS = {
    "air": {"groupName": "Air Combat"},
    "sur": {"groupName": "Ground Combat"},
}
WEAPONS = {
    "mg_l110": {
        "type": "Machine Guns",
        "weaponName": "L110",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_Minimi_PKG_Factory_MED-0e29fce7.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_Minimi_PKG_Factory_MED-0e29fce7.webp",
    },
    "smg_pw5a3": {
        "type": "SMG-PDWs",
        "weaponName": "PW5A3",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_MP5MLI_PKG_Factory_MED-2d1944b7.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_MP5MLI_PKG_Factory_MED-2d1944b7.webp",
    },
    "ar_m433": {
        "type": "Assault Rifles",
        "weaponName": "M433",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_HK433_PKG_Factory_MED-b06f02f7.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_HK433_PKG_Factory_MED-b06f02f7.webp",
    },
    "mg_rpkm": {
        "type": "Machine Guns",
        "weaponName": "RPKM",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_RPKM_PKG_Factory_MED-755b785f.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_RPKM_PKG_Factory_MED-755b785f.webp",
    },
    "sg_m87a1": {
        "type": "Shotguns",
        "weaponName": "M87A1",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_590A1_PKG_Factory_MED-4b387330.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_590A1_PKG_Factory_MED-4b387330.webp",
    },
    "pst_p18": {
        "type": "Pistols",
        "weaponName": "P18",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_M18_PKG_Factory_MED-88261bf8.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_M18_PKG_Factory_MED-88261bf8.webp",
    },
    "crb_x277": {
        "type": "Carbines",
        "weaponName": "M277",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_XM7_PKG_Factory_MED-26271094.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_XM7_PKG_Factory_MED-26271094.webp",
    },
    "ar_b36a4": {
        "type": "Assault Rifles",
        "weaponName": "B36A4",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_G36_PKG_Factory_MED-fb5466ec.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_G36_PKG_Factory_MED-fb5466ec.webp",
    },
    "ar_l85a3": {
        "type": "Assault Rifles",
        "weaponName": "L85A3",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_L85A3_PKG_Factory_MED-9bd0deaa.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_L85A3_PKG_Factory_MED-9bd0deaa.webp",
    },
    "snp_m2010": {
        "type": "Rifles",
        "weaponName": "M2010 ESR",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_M2010ESR_PKG_Factory_MED-f94fdae6.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_M2010ESR_PKG_Factory_MED-f94fdae6.webp",
    },
    "sg_m1014": {
        "type": "Shotguns",
        "weaponName": "M1014",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_M1014_PKG_Factory_MED-e011854a.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_M1014_PKG_Factory_MED-e011854a.webp",
    },
    "crb_ak205": {
        "type": "Carbines",
        "weaponName": "AK-205",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_AK205_PKG_Factory_MED-6d376081.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_AK205_PKG_Factory_MED-6d376081.webp",
    },
    "dmr_svk86": {
        "type": "DMRs",
        "weaponName": "SVK-8.6",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_SVCh_PKG_Factory_MED-bb06d385.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_SVCh_PKG_Factory_MED-bb06d385.webp",
    },
    "smg_sgx": {
        "type": "SMG-PDWs",
        "weaponName": "SGX",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_MPX_PKG_Factory_MED-e822f017.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_MPX_PKG_Factory_MED-e822f017.webp",
    },
    "dmr_lmr27": {
        "type": "DMRs",
        "weaponName": "LMR27",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_ARADMR_PKG_Factory_MED-dfe224de.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_ARADMR_PKG_Factory_MED-dfe224de.webp",
    },
    "crb_qbz192": {
        "type": "Carbines",
        "weaponName": "QBZ-192",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_QBZ192_PKG_Factory_MED-dc0a546b.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_QBZ192_PKG_Factory_MED-dc0a546b.webp",
    },
    "crb_m417a2": {
        "type": "Carbines",
        "weaponName": "M417 A2",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_HK417A2_PKG_Factory_MED-494ffe6e.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_HK417A2_PKG_Factory_MED-494ffe6e.webp",
    },
    "mg_drsiar": {
        "type": "Machine Guns",
        "weaponName": "DRS-IAR",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_M27IAR_PKG_Factory_MED-ae6675cd.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_M27IAR_PKG_Factory_MED-ae6675cd.webp",
    },
    "ar_kord6p67": {
        "type": "Assault Rifles",
        "weaponName": "KORD 6P67",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_6P67_PKG_Factory_MED-827b9414.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_6P67_PKG_Factory_MED-827b9414.webp",
    },
    "smg_usg90": {
        "type": "SMG-PDWs",
        "weaponName": "USG-90",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_P90_PKG_Factory_MED-2ab1db64.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_P90_PKG_Factory_MED-2ab1db64.webp",
    },
    "crb_m4a1": {
        "type": "Carbines",
        "weaponName": "M4A1",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_M4A1_PKG_Factory_MED-34529a82.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_M4A1_PKG_Factory_MED-34529a82.webp",
    },
    "mg_kts100mk8": {
        "type": "Machine Guns",
        "weaponName": "KTS100 MK8",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_Ultimax_PKG_Factory_MED-a8ebb482.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_Ultimax_PKG_Factory_MED-a8ebb482.webp",
    },
    "pst_m45a1": {
        "type": "Pistols",
        "weaponName": "M45A1",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_M45A1_PKG_Factory_MED-8c41691b.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_M45A1_PKG_Factory_MED-8c41691b.webp",
    },
    "smg_kv9": {
        "type": "SMG-PDWs",
        "weaponName": "KV9",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_Vector_PKG_Factory_MED-b5631ed7.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_Vector_PKG_Factory_MED-b5631ed7.webp",
    },
    "ar_sor556mk2": {
        "type": "Assault Rifles",
        "weaponName": "SOR-556 Mk2",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_SCARL_PKG_Factory_MED-b44467bb.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_SCARL_PKG_Factory_MED-b44467bb.webp",
    },
    "smg_pw7a2": {
        "type": "SMG-PDWs",
        "weaponName": "PW7A2",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_MP7A2_PKG_Factory_MED-406bc965.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_MP7A2_PKG_Factory_MED-406bc965.webp",
    },
    "mg_m123k": {
        "type": "Machine Guns",
        "weaponName": "M123K",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_MG4K_PKG_Factory_MED-cc064690.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_MG4K_PKG_Factory_MED-cc064690.webp",
    },
    "pst_m44": {
        "type": "Pistols",
        "weaponName": "M44",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_RagingHunter_PKG_Factory_MED-f9773307.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_RagingHunter_PKG_Factory_MED-f9773307.webp",
    },
    "mg_xm250": {
        "type": "Machine Guns",
        "weaponName": "M250",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_M250_PKG_Factory_MED-6e230a71.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_M250_PKG_Factory_MED-6e230a71.webp",
    },
    "smg_umg40": {
        "type": "SMG-PDWs",
        "weaponName": "UMG-40",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_UMP40_PKG_Factory_MED-22afa646.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_UMP40_PKG_Factory_MED-22afa646.webp",
    },
    "ar_tr7": {
        "type": "Assault Rifles",
        "weaponName": "TR-7",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_Tavor7_PKG_Factory_MED-e682f267.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_Tavor7_PKG_Factory_MED-e682f267.webp",
    },
    "smg_sl9": {
        "type": "SMG-PDWs",
        "weaponName": "SL9",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_APDW_PKG_Factory_MED-c4d1b829.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_APDW_PKG_Factory_MED-c4d1b829.webp",
    },
    "mg_m240l": {
        "type": "Machine Guns",
        "weaponName": "M240L",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_M240L_PKG_Factory_MED-1fa18e0c.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_M240L_PKG_Factory_MED-1fa18e0c.webp",
    },
    "smg_scw10": {
        "type": "SMG-PDWs",
        "weaponName": "SCW-10",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_APC10_PKG_Factory_MED-716a97fb.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_APC10_PKG_Factory_MED-716a97fb.webp",
    },
    "ar_nvo228e": {
        "type": "Assault Rifles",
        "weaponName": "NVO-228E",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_ACE32_PKG_Factory_MED-39a97220.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_ACE32_PKG_Factory_MED-39a97220.webp",
    },
    "mg_m60": {
        "type": "Machine Guns",
        "weaponName": "M/60",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_M60E6_PKG_Factory_MED-e600bb28.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_M60E6_PKG_Factory_MED-e600bb28.webp",
    },
    "snp_psr": {
        "type": "Rifles",
        "weaponName": "PSR",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_MRAD_PKG_Factory_MED-5035ce99.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_MRAD_PKG_Factory_MED-5035ce99.webp",
    },
    "dmr_svdm": {
        "type": "DMRs",
        "weaponName": "SVDM",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_SVDM_PKG_Factory_MED-b6c9aa50.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_SVDM_PKG_Factory_MED-b6c9aa50.webp",
    },
    "ar_ak4d": {
        "type": "Assault Rifles",
        "weaponName": "AK4D",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_G3A4_PKG_Factory_MED-0575cf00.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_G3A4_PKG_Factory_MED-0575cf00.webp",
    },
    "crb_sor300sc": {
        "type": "Carbines",
        "weaponName": "SOR-300SC",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_SCARSC_PKG_Factory_MED-7a6a50ff.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_SCARSC_PKG_Factory_MED-7a6a50ff.webp",
    },
    "crb_sg553r": {
        "type": "Carbines",
        "weaponName": "SG 553R",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_SIG553R_PKG_Factory_MED-afc2dfa8.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_SIG553R_PKG_Factory_MED-afc2dfa8.webp",
    },
    "crb_grtbc": {
        "type": "Carbines",
        "weaponName": "GRT-BC",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_MSBSGROTB_PKG_Factory_MED-25acf006.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_MSBSGROTB_PKG_Factory_MED-25acf006.webp",
    },
    "sg_185ksk": {
        "type": "Shotguns",
        "weaponName": "18.5KS-K",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_185KSK_PKG_Factory_MED-432c353f.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_MSBSGROTB_PKG_Factory_MED-25acf006.webp",
    },
    "pst_es57": {
        "type": "Pistols",
        "weaponName": "ES 5.7",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_FiveSeven_PKG_Factory_MED-c540cde7.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_FiveSeven_PKG_Factory_MED-c540cde7.webp",
    },
    "dmr_m39emr": {
        "type": "DMRs",
        "weaponName": "M39 EMR",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_M39EMR_PKG_Factory_MED-e17b5963.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_M39EMR_PKG_Factory_MED-e17b5963.webp",
    },
    "snp_sv98": {
        "type": "Rifles",
        "weaponName": "SV-98",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_SV98M_PKG_Factory_MED-c38f085a.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_SV98M_PKG_Factory_MED-c38f085a.webp",
    },
    "pst_m357trait": {
        "type": "Pistols",
        "weaponName": "M357 Trait",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_TRR8_PKG_Factory_MED-815434c3.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_TRR8_PKG_Factory_MED-815434c3.webp",
    },
    "pst_ggh22": {
        "type": "Pistols",
        "weaponName": "GGH-22",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_G22_PKG_Factory_MED-0faaf1e3.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_G22_PKG_Factory_MED-0faaf1e3.webp",
    },
    "snp_miniscout": {
        "type": "Rifles",
        "weaponName": "Mini Scout",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_MiniFix_PKG_Factory_MED-c8bab5ff.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_MiniFix_PKG_Factory_MED-c8bab5ff.webp",
    },
    "sg_db12": {
        "type": "Shotguns",
        "weaponName": "DB-12",
        "image": "https://cdn.gametools.network/weapons/bf6/T_UI_DP12_PKG_Factory_MED-0335554d.webp",
        "altImage": "https://cdn.gametools.network/weapons/bf6/white/T_UI_DP12_PKG_Factory_MED-0335554d.webp",
    },
}
WEAPON_GROUPS = {
    "snp": {"groupName": "Rifles"},
    "crb": {"groupName": "Carbines"},
    "mg": {"groupName": "Machine Guns"},
    "sg": {"groupName": "Shotguns"},
    "ar": {"groupName": "Assault Rifles"},
    "smg": {"groupName": "SMG-PDWs"},
    "dmr": {"groupName": "DMRs"},
}
GADGETS = {
    "callin_airstrike": {
        "type": "Strike Packages",
        "gadgetName": "Air Strike",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_CALLINS_Airstrike_Thumb-638b4ee1.png",
    },
    "mpaps": {
        "type": "Deployable Gadgets",
        "gadgetName": "MP-APS",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_MPAPS_Icon-b735994d.png",
    },
    "gpdis": {
        "type": "Deployable Gadgets",
        "gadgetName": "GPDIS",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_EIDOS_Icon-2273a3ee.png",
    },
    "c4": {
        "type": "Explosives",
        "gadgetName": "C-4 Explosives",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_C4_Icon-330d6ee2.png",
    },
    "gl_he": {
        "type": "Grenade Launchers",
        "gadgetName": "M320A1 HE",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_M320_HE_Icon-83b61ed0.png",
    },
    "rl_ungui": {
        "type": "Launchers",
        "gadgetName": "RPG-7V2",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_RPG7v2_Icon-ce5453fd.png",
    },
    "rl_aimgui": {
        "type": "Launchers",
        "gadgetName": "M136 AT",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_AT4_Icon-beadb575.png",
    },
    "gren_frag": {
        "type": "Grenade",
        "gadgetName": "Frag Grenade",
        "image": "",
    },
    "mine_press": {
        "type": "Explosives",
        "gadgetName": "M15",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_AT_Mine_Icon-2288ec8e.png",
    },
    "gl_tb": {
        "type": "Grenade Launchers",
        "gadgetName": "M320A1 THRM",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_M320_Thermobaric_Icon-3f13c762.png",
    },
    "repair": {
        "type": "",
        "gadgetName": "Repair Tool",
        "image": "",
    },
    "gren_inc": {
        "type": "Grenade",
        "gadgetName": "Incendiary Grenade",
        "image": "",
    },
    "ladder": {
        "type": "",
        "gadgetName": "Assault Ladder",
        "image": "",
    },
    "sup_bag": {
        "type": "",
        "gadgetName": "Supply Bag",
        "image": "",
    },
    "deplocover": {
        "type": "",
        "gadgetName": "Deployable Cover",
        "image": "",
    },
    "gren_stun": {
        "type": "",
        "gadgetName": "Stun Grenade",
        "image": "",
    },
    "gren_smoke": {
        "type": "",
        "gadgetName": "Smoke Grenade",
        "image": "",
    },
    "gren_flash": {
        "type": "",
        "gadgetName": "Flash Grenade",
        "image": "",
    },
    "recondrone": {
        "type": "",
        "gadgetName": "Recon Drone",
        "image": "",
    },
    "laserdes": {
        "type": "",
        "gadgetName": "Laser Designator",
        "image": "",
    },
    "tugs": {
        "type": "",
        "gadgetName": "Motion Sensor",
        "image": "",
    },
    "rl_longra": {
        "type": "Launchers",
        "gadgetName": "MAS 148 Glaive",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_Javelin_Icon-a0ce2091.png",
    },
    "mine_ap": {
        "type": "Explosives",
        "gadgetName": "M18A1",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_Claymore_Icon-5c7f1590.png",
    },
    "adren": {
        "type": "",
        "gadgetName": "Adrenaline Injector",
        "image": "",
    },
    "mine_tws": {
        "type": "Explosives",
        "gadgetName": "M4A1 Slam",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_M4_SLAM_Icon-541f624c.png",
    },
    "eodbot": {
        "type": "",
        "gadgetName": "EOD Bot",
        "image": "",
    },
    "gl_breach": {
        "type": "Grenade Launchers",
        "gadgetName": "X95 BRE",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_Drill_Launcher_Icon-1e1481d9.png",
    },
    "mine_mosen": {
        "type": "Explosives",
        "gadgetName": "PTKM-1R",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_PTKM_1R_Icon-ceb07fc0.png",
    },
    "gren_mfrag": {
        "type": "",
        "gadgetName": "Mini Grenade",
        "image": "",
    },
    "rl_surtoair": {
        "type": "Launchers",
        "gadgetName": "SLM-93A",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_Stinger_Icon-d4590623.png",
    },
    "il_airburst": {
        "type": "Grenade Launchers",
        "gadgetName": "SICH G1 WP",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_Airburst_Incendiary_Launcher_Icon-da8af82a.png",
    },
    "vehsupcra": {
        "type": "",
        "gadgetName": "Supply Crate",
        "image": "",
    },
    "tknife": {
        "type": "",
        "gadgetName": "Throwing Knife",
        "image": "",
    },
    "msensor": {
        "type": "",
        "gadgetName": "Proximity Detector",
        "image": "",
    },
    "deployable": {
        "type": "",
        "gadgetName": "Deployable Cover",
        "image": "",
    },
    "deploymortar": {
        "type": "Class Gadgets",
        "gadgetName": "LWCMS",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_Deployable_Mortar_Icon-97b13652.png",
    },
    "deploybeacon": {
        "type": "Deployable Gadgets",
        "gadgetName": "QLINK 6",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_Spawn_Beacon_Icon-b6e67f1c.png",
    },
    "decoy": {
        "type": "",
        "gadgetName": "Sniper Decoy",
        "image": "",
    },
    "gl_smoke": {
        "type": "Grenade Launchers",
        "gadgetName": "M320A1 SMK",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_M320_Smoke_Icon-dfa762d9.png",
    },
    "callin_wpdrop": {
        "type": "Strike Packages",
        "gadgetName": "Weapon Drop",
        "image": "https://cdn.gametools.network/gadgets/bf6/U_Gadget_WeaponDrop_Granite-2c6c3b16.webp",
    },
    "callin_uav": {
        "type": "Strike Packages",
        "gadgetName": "UAV Overwatch",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_CALLINS_Uav_Thumb-fdf9a78e.png",
    },
    "callin_resupply": {
        "type": "Strike Packages",
        "gadgetName": "",
        "image": "",
    },
    "callin_antiveh": {
        "type": "Strike Packages",
        "gadgetName": "Anti-Vehicle Drop",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_CALLINS_SupAnti_Thumb-84fc98fc.png",
    },
    "callin_smoke": {
        "type": "Strike Packages",
        "gadgetName": "Smoke Screen",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_CALLINS_Smoke_Thumb-5d515d26.png",
    },
    "callin_artstrike": {
        "type": "Strike Packages",
        "gadgetName": "Artillery Strike",
        "image": "https://cdn.gametools.network/gadgets/bf6/T_UI_CALLINS_Artillery_Thumb-126cad71.png",
    },
}

GADGET_GROUPS = {
    "gl": {"groupName": "Grenade Launchers"},
    "mine": {"groupName": "Explosives"},
    "grenades": {"groupName": "Grenades"},
    # "rl": {"groupName": ""},
    # "il": {"groupName": ""},
    "callins": {"groupName": "Call-ins"},
}



from global_mapping.readability.bf2042.saved_items.singleton import Singleton
from . import (
    ar_sa,
    de_de,
    en_us,
    es_es,
    es_mx,
    fr_fr,
    it_it,
    ja_jp,
    ko_kr,
    pl_pl,
    ru_ru,
    zh_cn,
    zh_tw,
    pt_br,
)


class LangSwitch(metaclass=Singleton):
    __langs: dict = {
        "ar-sa": ar_sa.translation,
        "de-de": de_de.translation,
        "en-us": en_us.translation,
        "es-es": es_es.translation,
        "es-mx": es_mx.translation,
        "fr-fr": fr_fr.translation,
        "it-it": it_it.translation,
        "ja-jp": ja_jp.translation,
        "ko-kr": ko_kr.translation,
        "pl-pl": pl_pl.translation,
        "ru-ru": ru_ru.translation,
        "zh-cn": zh_cn.translation,
        "zh-tw": zh_tw.translation,
        "pt-br": pt_br.translation,
    }

    async def updateLanguage(self, lang: str, translation: dict):
        current_lang = self.__langs.get(f"{lang}", None)
        if current_lang is not None:
            self.__langs[lang] = translation

    async def get(self, lang: str):
        return self.__langs.get(f"{lang}".lower(), self.__langs["en-us"])


async def selectLanguage(lang: str):
    switcher = LangSwitch()
    return await switcher.get(lang)
